/*
    Fichier chargepartiel.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence-fr.txt

    Traductions des droits d'utilisation dans les fichiers :
    licence-de.txt , licence-en.txt , licence-es.txt ,
    licence-it.txt , licence-nl.txt , licence-pt.txt ,
    licence-eo.txt , licence-eo-utf.txt

    Droits d'utilisation galement sur la page web :
    http://libremail.tuxfamily.org/voir.php?page=droits


    Ce programme tlcharge parmi les mail en attente sur le serveur
    de messagerie, ceux dont l'un des champs contient (ou ne contient
    pas) une chaine de caractres particulire.

    Un fichier de configuration est utilis pour se connecter  la
    boite aux lettres et pour fixer le rpertoire racine du systme
    de messagerie.

    Les mail entrants sont stocks dans le sous rpertoire "entree"
    de ce rpertoire racine et se distinguent par leur numro.
*/

#define appli   // pour la dclaration de variables globales  l'application

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include "messages.h"
#include "buflect.h"
#include "ficonf.h"
#include "pop.h"
#include "numail.h"
#include "trtentete.h"
#include "szchemin.h"


/* constantes pour indiquer si l'on efface le(s) mail(s) tlcharg(s) */
#define OPTDEFAUT  0
#define EFFACE     1
#define CONSERVE   2


/* prototypes */
int  testchamp (char *nomchamp, char *chainetest, int numes);
void chargemail (int numes, long numail, int effacement);


/* variable globale au source
   (pour viter des tonnes de passages de paramtres) */

char dirmails [szchemin]; // rpertoire de destination des messages

// chaines de caractres mmorises pour viter appels rptitifs  message ()
char mess_analys [50]; // message signalant l'analyse d'un mail
char mess_telech [50]; // message signalant le tlchargement d'un mail



/* programme principal */

int main (int nbarg, char *varg[])
{
    FILE *fconf;       // descripteur du fichier de configuration
    int  numes, nbmes; // numro du mail courant et nombre de mails
    int  nbrecup = 0;  // nombre de mails tlchargs
    long numail;       // numro du mail dans le fichier tlcharg
    int  option;       // option : dtermine si on conserve les mails rcuprs
    int  optn;         // option : mail rcupr si la chaine n'est pas
                       //          trouve dans le champ ou si champ absent
    char *ncom;        // nom de la commande que l'on excute (chargepartiel)


    // rcupration du nom de l'excutable
    memcom (*varg);

    // se positionner sur le premier argument de chargepartiel
    varg ++;
    nbarg --;

    // options de fonctionnement par dfaut
    option = OPTDEFAUT;
    optn = 0;

    // rcupration de l'option de fonctionnement ventuelle
    while (nbarg > 0 && **varg == '-')
    {
        switch (varg [0][1])
        {
            // option effacement du serveur des mails rcuprs
            case 'd': if (option == CONSERVE)
                      {
                          // "Options -d et -k incompatibles"
                          affiche_err ("COMPAT_OPT_DK");
                          return (-1);
                      }
                      else
                          option = EFFACE;

                      break;

            // option conservation sur le serveur des mails rcuprs
            case 'k': if (option == EFFACE)
                      {
                          // "Options -d et -k incompatibles"
                          affiche_err ("COMPAT_OPT_DK");
                          return (-1);
                      }
                      else
                          option = CONSERVE;

                      break;

            // option rcupration du mail si chaine non trouve
            case 'n': optn = 1;
                      break;

                      // "Option %s incorrecte"
            default : aff_err_arg ("ERR_OPTION", *varg);
                      nbarg = 0;   // pour afficher la syntaxe de la commande
        }

        // dcompter l'argument trait
        varg ++;
        nbarg --;
    }

    // controle du nombre d'arguments
    if (nbarg == 3)
    {
        // ouvrir le fichier de configuration
        fconf = ouvre_ficonf (varg [2]);

        if (fconf)
        {
            // connexion sur le compte mail du serveur pop
            if (connect_pop (fconf))
            {
                // rcuprer le rpertoire racine de la messagerie
                fgets (dirmails, sizeof (dirmails) - 7, fconf);
                dirmails [strlen (dirmails) - 1] = '\0';

                // si ce rpertoire existe
                if (access (dirmails, 0) == 0)
                {
                    // crer si ncessaire le sous repertoire d'entre
                    strcat (dirmails, "/");
                    strcat (dirmails, ficdir ("DIR_ENTREE"));
                    mkdir (dirmails, 0755);

                    // rcupration du nombre de mails
                    nbmes = nbmails ();

                    // si des mails peuvent tre rcuprs
                    if (nbmes)
                    {
                        // rcupration du numro du dernier fichier mail
                        numail = num_dermail (dirmails);

                        // Initialisation des messages  afficher  chaque
                        // analyse ou tlchargement de mail

                        // "\rAnalyse du mail n %d       "
                        strcpy (mess_analys, message ("ANALYSE_MAIL_BL"));

                        // "\rTlchargement du mail n %d"
                        strcpy (mess_telech, message ("TELECH_MAIL"));

                        // pour chaque fichier mail du serveur
                        for (numes = 1; numes <= nbmes; numes++)
                        {
                            // si le champ  tester contient la chaine
                            // ou ne la contient pas si option -n
                            if (testchamp (varg [0], varg [1], numes) != optn)
                            {
                                // tlcharger le mail correspondant
                                chargemail (numes, ++numail, option);

                                // on compte les mails tlchargs
                                nbrecup ++;
                            }
                        }

                        // dition d'un rcapitulatif
                        // "\n%d mails tlchargs sur %d\n"
                        printf (message ("TOTAL_TELECH"), nbrecup, nbmes);

                        // sauvegarde du numro du dernier fichier mail
                        sauv_num_dermail (dirmails, numail);
                    }
                }
                else
                    // "rpertoire de messagerie %s inexistant"
                    aff_err_arg ("REP_RACINE_ABSENT", dirmails);

                // se dconnecter proprement du serveur pop
                deconnect_pop ();
            }

            // on n'a plus besoin du fichier de configuration
            fclose (fconf);
        }
    }
    else
    // "Syntaxe : %s [(-k|d)] [-n] nom_champ chaine_champ fichier_configuration"
        psyntaxe ("SYNT_CHARGEPARTIEL");

    // pour faire plaisir  gcc qui veut une fonction main de type int
    return (0);
}



/* vrifie si le champ nomchamp du mail de numro numes
   contient la chaine passe en 2me paramtre */

int testchamp (char *nomchamp, char *chainetest, int numes)
{
    char bufw [80];     // buffer d'envoi d'une requte
    char valchamp [80]; // contenu du champ  tester
    int  poschamp;      // positionnement dans valchamp
    int  i;             // positionnement dans chainetest


    // initialisation
    valchamp [0] = '\0';

    // message de suivi de droulement
    // "\rAnalyse du mail n %d       "
    printf (mess_analys, numes);
    fflush (stdout);

    // demande de lecture de l'entte du message
    sprintf (bufw, "TOP %d 1", numes);
    env_pop (bufw);

    // lire la premire ligne de l'entte du message
    lire_pop ();

    // termin pour ce mail si erreur d'envoi cot serveur
    if (memcmp (buf_lect, "-ERR ", 5) == 0)
    {
        // Erreur serveur pour l'accs au mail
        aff_err_argnum ("ERREUR_SERVEUR", numes);
        return 0;
    }

    // lecture et mmorisation des caractristiques du message
    do
    {
        // si elle contient le champ recherch
        if (start (nomchamp))
        {
            // convertir les caractres spciaux
            majlignentete ();

            // et mmoriser ce champ
            membuf (valchamp);
        }

        // lire la ligne suivante de l'entte du message
        lire_pop ();
    }
    // lecture termine si ligne limite  un .
    while (buf_lect [0] != '.' || buf_lect [1] != '\0');

    // on se positionne au dbut du champ  tester
    poschamp = 0;

    // vrifier si la chaine chainetest se trouve dans valchamp
    do
    {
        i = 0;

        while (valchamp [poschamp+i] == chainetest [i])
            // termin si tous les caractres de chainetest trouvs
            if (chainetest [++i] == 0)
                return 1;
    }
    // sinon recherche dans la suite de valchamp
    while (valchamp [poschamp++]);

    // aucune des chaines de la liste n'a t trouve dans valchamp
    return (0);
}



/* tlcharge le message choisi et l'enlve du serveur si c'est demand */

void chargemail (int numes, long numail, int effacement)
{
    char bufw [80];    // buffer d'envoi d'une requte
    char ficmail [sizeof (dirmails) + 9];
    FILE *fmail;


    // cration du fichier mail
    sprintf (ficmail, "%s/r%07ld.n", dirmails, numail);

    fmail = fopen (ficmail, "w");

    if (fmail)
    {
        // message de suivi du droulement
        // "\rTlchargement du mail n %d"
        printf (mess_telech, numes);
        fflush (stdout);

        // demande de lecture du message et rception de l'acquittement
        sprintf (bufw, "RETR %d", numes);
        env_pop (bufw);
        lire_pop ();

        // termin pour ce mail si erreur d'envoi cot serveur
        if (memcmp (buf_lect, "-ERR ", 5) == 0)
        {
            // Erreur serveur pour l'accs au mail
            aff_err_argnum ("ERREUR_SERVEUR", numes);
            fclose (fmail);
            unlink (ficmail);
            return;
        }

        // lire la premire ligne de l'entte du mail
        lire_pop ();

        // lecture et copie du message jusqu' une ligne limite a un .
        while (buf_lect [0] != '.' || buf_lect [1] != '\0')
        {
            // copie de la ligne lue dans le fichier mail
            fputs (buf_lect, fmail);
            fputc ('\n', fmail);     // ncessaire car supprim de buf_lect

            // et lecture de la suivante
            lire_pop ();
        }

        // fermer le fichier contenant le mail
        fclose (fmail);

        // et supprimer si ncessaire le mail du serveur
        if (effacement != CONSERVE)
        {
            sprintf (bufw, "DELE %d", numes);
            env_pop (bufw);
            lire_pop ();
        }
    }
    else
        // "Impossible d'crire dans le fichier %s"
        aff_err_arg ("IMPOS_ECR_FICH", ficmail);
}
