/*
    Fichier deplpartaille.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence-fr.txt

    Traductions des droits d'utilisation dans les fichiers :
    licence-de.txt , licence-en.txt , licence-es.txt ,
    licence-it.txt , licence-nl.txt , licence-pt.txt ,
    licence-eo.txt , licence-eo-utf.txt

    Droits d'utilisation galement sur la page web :
    http://libremail.tuxfamily.org/voir.php?page=droits


    Ce programme dplace vers une autre boite aux lettres parmi les
    mail en attente sur le serveur de messagerie, ceux dpassant la
    taille limite ou n'atteignant pas la taille minimale passe en
    paramtre.

    Si la taille indique est positive, il s'agit de la taille
    maximale au del de laquelle le mail est dplac.
    Si la taille indique est ngative, il s'agit de la taille
    minimale en de de laquelle le mail est dplac.

    Une option permet de conserver les mails slectionns dans la
    boite aux lettres initiale. Dans ce cas, les mails slectionns
    sont seulement copis.

    Un fichier de configuration est utilis pour se connecter 
    la boite aux lettres, pour connaitre l'adresse d'expdition
    et pour slectionner le serveur SMTP servant  la rexpdition
    des mails.
    Il s'agit d'une variante des fichiers de configuration utiliss
    par la plupart des autres outils de libremail.
*/

#define appli   // pour la dclaration de variables globales  l'application

#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "messages.h"
#include "buflect.h"
#include "ficonf.h"
#include "pop.h"
#include "smtp.h"
#include "copiedepl.h"


/* constante */

// nombre de mails qu'on peut dplacer en une seule passe
#define szmaxliste    100


/* prototypes */
void selectmails (int numdepart);
void copieliste ();
void deplaceliste ();


/* variables globales au source
   (pour viter des tonnes de passages de paramtres) */

long taillepivot;    // taille minimale ou maximale des mails dplacs
int  listemail [szmaxliste];  // numro des gros mails  tester
int  szlistemail;    // nombre d'lments dans listemail


/* programme principal */

int main (int nbarg, char *varg[])
{
    char serv_smtp [80]; // nom du serveur smtp utilis
    FILE *fconf;         // descripteur du fichier de configuration
    long numail;         // numro du mail  dplacer ou copier
    int  optc;           // option : on conserve les mails dplacs


    // rcupration du nom de l'excutable
    memcom (*varg);

    // nombre d'arguments passs en paramtre
    nbarg--;

    // si option -c (copie des mails plutt que dplacement)
    if (nbarg > 1 && strcmp (varg [1], "-c") == 0)
    {
        // mmoriser cette option
        optc = 1;

        // sauter l'argument trait
        varg ++;
        nbarg --;
    }
    // sinon, les mails choisis seront dplacs
    else
        optc = 0;

    // controle du nombre d'arguments restants
    if (nbarg == 2)
    {
        // rcuprer la taille pivot des mails
        // accepts pour n'importe quel expditeur
        taillepivot = atol (varg [1]);

        // erreur de syntaxe si la taille pivot est nulle
        if (! taillepivot)
        {
            // "Paramtre taille incorrect ou mal plac"
            affiche_err ("ERR_TAILLE");
            return (0);
        }

        // cette taille peut tre exprime en koctets
        if (tolower (varg [1][strlen (varg [1]) - 1]) == 'k')
            taillepivot *= 1024;

        // ouvrir le fichier de configuration
        fconf = ouvre_ficonf (varg [2]);

        // si on a pu y accder
        if (fconf)
        {
            // connexion sur le compte mail du serveur pop
            if (connect_pop (fconf))
            {
                // rcuprer et mmoriser les adresses
                // Email d'expdition et de destination
                mem_adr (adr_exped, fconf);
                mem_adr (adr_dest, fconf);

                // slectionner les mails  dplacer en fonction de leur taille
                selectmails (0);

                // s'il y a des mails  dplacer
                if (szlistemail)
                {
                    // recuprer le nom du serveur smtp
                    fgets (serv_smtp, 80, fconf);
                    serv_smtp [strlen (serv_smtp) - 1] = '\0';

                    // connexion au serveur smtp pour l'envoi des mails
                    if (connect_smtp (serv_smtp))
                    {
                        // dplacer ou copier les mails slectionns
                        if (optc)
                            copieliste ();
                        else
                            deplaceliste ();

                        // si la liste des mails slectionns avait atteint sa
                        // taille maximale, on relance une ou plusieurs tape(s)
                        // de filtrage
                        while (szlistemail == szmaxliste)
                        {
                            // slectionner d'autres mails  dplacer
                            selectmails (listemail [szlistemail - 1]);

                            // s'il y en a
                            if (szlistemail)
                            {
                                // dplacer ou copier les mails slectionns
                                if (optc)
                                    copieliste ();
                                else
                                    deplaceliste ();
                            }
                        }

                        // pour l'affichage
                        putchar ('\n');

                        // fermeture de la connexion smtp
                        deconnect_smtp ();
                    }
                }
                // sinon, aucun mail  dplacer ou copier
                else
                {
                    // message adapt  l'opration demande
                    if (taillepivot > 0)
                        // "Aucun mail ne dpasse la taille limite"
                        affiche_msg ("AUCUN_GROSMAIL");
                    else
                        // "Aucun mail n'est plus petit que la taille choisie"
                        affiche_msg ("AUCUN_PETITMAIL");
                }

                // se dconnecter proprement du serveur pop
                deconnect_pop ();
            }

            // on n'a plus besoin du fichier de configuration
            fclose (fconf);
        }
    }
    else
        // "Syntaxe : %s [-c] taille fichier_configuration"
        psyntaxe ("SYNT_DEPLPARTAILLE");

    // pour faire plaisir  gcc qui veut une fonction main de type int
    return (0);
}


/* mmorise les numros des mails  dplacer en fonction
   de leur taille et dont le numro est > numdepart */

void selectmails (int numdepart)
{
    long numail;     // numro du mail dont on liste la taille
    long taillemail; // taille de ce mail


    // initialisation
    szlistemail = 0;

    // mmoriser les tailles des mails
    env_pop ("LIST");

    // sauter le message d'entte de la liste
    lire_pop ();

    // lire la taille d'un mail
    lire_pop ();

    // tantque des mails restent  analyser
    while (buf_lect [0] != '.')
    {
        // si ce mail n'a pas t encore examin comme mail  dplacer
        // et que la liste des mails  dplacer n'est pas pleine
        if (numail > numdepart && szlistemail < szmaxliste)
        {
            // rcuprer la taille du mail
            sscanf (buf_lect, "%ld%ld", &numail, &taillemail);

            // astuce pour simplifier le test qui suit
            if (taillepivot < 0)
                taillemail = - taillemail;

            // si ce mail dpasse la taille maximale (taillepivot > 0)
            // ou n'atteint pas la taille minimale (taillepivot < 0)
            if (taillemail > taillepivot)
            {
                // mmoriser le numro de ce mail
                listemail  [szlistemail ++] = numail;
            }
        }

        // lire la taille du mail suivant
        lire_pop ();
    }
}


/* rexpdie les mails slectionns vers la nouvelle adresse
   email sans les supprimer sur le serveur courant */

void copieliste ()
{
    int numail;  // numro de mail  copier
    int i;       // compteur


    // pour tous les mails slectionns
    for (i = 0; i < szlistemail; i++)
    {
        // rcuprer le numro du mail
        numail = listemail [i];

        // "Copie du mail n %d"
        printf (message ("COPIE_MAIL"), numail);
        fflush (stdout);

        // envoyer une copie du mail  l'autre adresse
        copiemail (listemail [i]);
    }
}


/* rexpdie les mails slectionns vers la nouvelle adresse
   email et les supprime du serveur courant */

void deplaceliste ()
{
    char bufw [80]; // buffer d'envoi d'une requte de destruction
    int numail;     // numro de mail  copier
    int i;          // compteur


    // pour tous les mails slectionns
    for (i = 0; i < szlistemail; i++)
    {
        // rcuprer le numro du mail
        numail = listemail [i];

        // "Dplacement du mail n %d"
        printf (message ("DEPL_MAIL"), numail);
        fflush (stdout);

        // envoyer une copie du mail  l'autre adresse
        copiemail (listemail [i]);

        // demande de destruction du mail sur le serveur initial
        detruitmail (listemail [i]);
    }
}
