/*
    Fichier recuppj.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence-fr.txt

    Traductions des droits d'utilisation dans les fichiers :
    licence-de.txt , licence-en.txt , licence-es.txt ,
    licence-it.txt , licence-nl.txt , licence-pt.txt ,
    licence-eo.txt , licence-eo-utf.txt

    Droits d'utilisation galement sur la page web :
    http://libremail.tuxfamily.org/voir.php?page=droits


    Ce programme rcupre les fichiers joints contenu dans
    le fichier mail dont le nom est pass en paramtre.
    Le chemin d'accs au fichier peut tre absolu ou relatif

    Les fichiers joints sont stocks :
    1) si le rpertoire racine du systme de messagerie est connu
       (commande recuppj lance depui la suite vmail...)
       - dans le rpertoire prcis dans le fichier <racine>/dirpj
       -  dfaut dans le rpertoire <racine>/pjointes
    2) dans le cas contraire, dans un rpertoire choisi par l'utilisateur
*/

#define appli   // pour la dclaration de variables globales  l'application

#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>
#include "messages.h"
#include "buflect.h"
#include "fmail.h"
#include "encodage.h"
#include "trtentete.h"
#include "trtligne.h"
#include "trtbordure.h"
#include "base64.h"


/* prototypes */
void extraire ();
void recup_b64 ();
void recup_quot ();
void recup_brut ();
void choixdirpj ();


/* variables globales au source
   (pour viter des tonnes de passages de paramtres) */

FILE *ficdest;       // descripteur du fichier rcupr
char chemfich [120]; // chemin d'accs au fichier joint


/* programme principal */

int main (int nbarg, char *varg[])
{
    // rcupration du nom de l'excutable
    memcom (*varg);

    // controle du nombre d'arguments
    if (--nbarg == 1)
    {
        // ouvrir le fichier mail
        fmail = fopen (varg [1], "r");

        // si le fichier a pu tre ouvert
        if (fmail)
        {
            // extraire les pices jointes
            extraire ();

            // et fermer le fichier mail
            fclose (fmail);
        }
        else
            // "Fichier %s non trouv"
            aff_err_arg ("FICH_ABSENT", varg [1]);
    }
    else
        // "Syntaxe : %s nom_fichier_mail"
        psyntaxe ("SYNT_GENE_FICMAIL");

    // pour faire plaisir  gcc qui veut une fonction main de type int
    return (0);
}


/* lit le message choisi et rcupre les fichiers joints si
   le mail est en mode multipart/mixed ou multipart/related */

void extraire ()
{
    int  dansmail;     // pour s'arrter de lire si fin de fichier mail
    int  posjoint;     // position du nom du fichier joint
    int  posdebnom;    // position du nom de fichier dans son chemin d'accs
    int  poschem;      // position dans le chemin d'accs au fichier
    int  ctype;        // pour mmoriser champ content-type du fichier
    int  derctype;     // pour mmoriser champ content-type de la section
    char reponse [10]; // rponse  une question


    // lecture du fichier mail jusqu'au champ Content-Type
    while (lire_fmail () && ! start ("Content-Type"))
        ;

    // rcuprer le type de base du fichier mail
    ctype = recup_ctype ();

    // si fichier en mode multipart/mixed ou multipart/related
    if (ctype == MultipMixed || ctype == MultipRel)
    {
        // retour  la configuration standard du clavier si ncessaire
        system ("stty icanon echo");

        // rcuprer ou choisir le rpertoire de destination des pices jointes
        choixdirpj ();

        //le complter par un  /
        posdebnom = strlen (chemfich);
        chemfich [posdebnom++] = '/';
        chemfich [posdebnom]  = '\0';

        // mmoriser la bordure
        while (*buf_lect && ! mem_boundary ())
            dansmail = lire_fmail ();

        // boucle de rcupration de fichiers joints
        while (dansmail && nbordures)
        {
            // initialisation
            chemfich [posdebnom] = '\0';

            // se positionner sur la prochaine section si on n'y est pas dj
            while (dansmail && ! surbordure ())
                dansmail = lire_fmail ();

            // initialisation du type de la section courante
            derctype = 0;

            // initialisation du mode d'encodage du fichier
            encodage_texte = defaut;

            // chercher et afficher le nom du fichier joint s'il y en a un
            do
            {
                // lire une ligne et la convertir
                dansmail = lire_fmail ();

                // mmoriser le type de la section courante
                if (start ("Content-Type"))
                    derctype = recup_ctype ();

                // chercher la position du nom de fichier
                posjoint = posnomfic ();

                // si trouv et pas une section text/html
                if (posjoint && derctype != TextHtml)
                {
                    // convertir les donnes encodes comme dans une entte
                    majlignentete ();

                    // rcuprer son nom
                    poschem = posdebnom;

                    // sans les (horribles) blancs qu'il pourrait contenir !!!
                    while (buf_lect [posjoint] != '"' && buf_lect [posjoint])
                    {
                        // pas de blanc dans un nom de fichier !!!
                        if (buf_lect [posjoint] != ' ')
                            chemfich [poschem++] = buf_lect [posjoint];

                        posjoint++;
                    }

                    // terminer le nom de fichier
                    chemfich [poschem] = '\0';
                }
                // mmoriser aussi le mode d'encodage
                else if (start ("Content-Transfer-Encoding"))
                    mem_encodage ();
            }
            while (dansmail && buf_lect [0] != '\0');

            // si la section concerne un fichier joint
            if (chemfich [posdebnom])
            {
                // demande de confirmation si le fichier existe dj
                if (access (chemfich, 0) == 0)
                {
                    // "Fichier %s dj existant.\n"
                    // "Voulez vous le remplacer ? "
                    printf (message ("FICH_EXISTANT"), chemfich);
                    affiche_msg_nocr ("MAJ_FICJOINT");

                    // lire et traiter la rponse
                    fgets (reponse, sizeof (reponse), stdin);

                    if (tolower (*reponse) != 'n')
                        ficdest = fopen (chemfich, "w");
                    else
                        ficdest = NULL;
                }
                else
                {
                    // sinon crer le fichier
                    ficdest = fopen (chemfich, "w");

                    // indicateur pour test ultrieur
                    *reponse = 'o';
                }

                // si l'ouverture s'est bien passe
                if (ficdest)
                {
                    // rcuprer le fichier en tenant compte du mode d'encodage
                    if (encodage_texte == Base64)
                        recup_b64 ();

                    else if (encodage_texte == QuotedPrint)
                        recup_quot ();
                    else
                        recup_brut ();

                    // rcupration termine
                    fclose (ficdest);
                }
                // sinon message d'erreur ventuel
                else if (tolower (*reponse) != 'n')
                    // "Le fichier %s n'a pu tre rcupr"
                    aff_err_arg ("ERR_RECUP_PJ", chemfich);
            }
        }
    }
    else
        // "Ce fichier ne contient pas de pices jointes"
        affiche_err ("MAIL_SANS_PJ");
}


/* rcupre un fichier joint encod base64 */

void recup_b64 ()
{
    int nbcar; // nombre de caractres aprs supprssion de l'encodage base64
    int i;     // compteur


    // tant que ligne vide
    while (*buf_lect == '\0')
        // lire la ligne suivante
        lire_fmail ();

    // rpter
    do
    {
        // convertion les donnes de la ligne
        nbcar = decode64 (buf_lect);

        // recopier les caractres correspondants dans le fichier
        for (i = 0; i < nbcar; i++)
            fputc (buf_lect [i], ficdest);
    }
    // passer  la ligne suivante jusqu' ligne vide ou bordure
    while (lire_fmail () && *buf_lect && !surbordure ());
}


/* rcupre un fichier joint encod quoted-printable

   Remarque importante : on n'utilise pas majligne pour la conversion
   en effet, majligne filtre les caractres de controle pour limiter
   les problmes d'affichage, ce qu'il faut viter de faire pour
   rcuprer un fichier joint */

void recup_quot ()
{
    char *carcourant;  // caractres de buf_lect


    // tant que non fin de section, lire une ligne
    while (lire_fmail () && !surbordure ())
    {
        // positionnement en dbut de buffer
        carcourant = buf_lect;

        // tant que non fin de ligne
        while (*carcourant)
        {
            // si caractre ordinaire
            if (*carcourant != '=')
            {
                // on le recopie
                fputc (*carcourant, ficdest);

                // et on passe au suivant
                carcourant ++;
            }
            // sinon
            else
            {
                // avancer d'un caractre
                carcourant ++;

                // si on n'est pas en fin de ligne
                if (*carcourant)
                {
                    // convertir le code hexa et le recopier dans le fichier
                    fputc (hexa (carcourant), ficdest);

                    // se positionner aprs le code hexa
                    carcourant = carcourant + 2;
                }
            }
        }

        // rajout d'un ventuel CR LF
        if (*(carcourant - 1) != '=')
        {
            fputc ('\r', ficdest);
            fputc ('\n', ficdest);
        }

        // on traitera de la mme manire les autres lignes
    }
}


/* rcupre un fichier joint non encod */

void recup_brut ()
{
    char *carcourant;    // caractres de buf_lect
    int  sautligne = 0;  // mmorise les sauts de ligne en attente


    // tant que non fin de section, lire une ligne
    while (lire_fmail () && !surbordure ())
    {
        // si ce n'est pas la premire ligne
        if (sautligne)
        {
            // terminer la ligne prcdente par un caractre LF
            fputc ('\n', ficdest);

            // inutile de remettre sautligne  0
        }

        // positionnement en dbut de buffer
        carcourant = buf_lect;

        // tant que non fin de ligne
        while (*carcourant)
        {
            // on recopie le caractre lu
            fputc (*carcourant, ficdest);

            // et on passe au suivant
            carcourant ++;
        }

        // un saut de ligne sera ncessaire si d'autres lignes suivent celles-ci
        sautligne = 1;

        // on traitera de la mme manire les autres lignes
    }
}


/* rcupre le nom du rpertoire de destination des pices jointes */

void choixdirpj ()
{
    FILE *fic_dirpj;   // descripteur fichier contenant nom rpertoire pices
    char reponse [10]; // rponse  une question


    // si on connait le rpertoire racine des mails
    // (ce qui est le cas lorsque le sous rpertoire d'envoi est connu)
    if (getenv ("mailenv"))
    {
        // rcuprer ce rpertoire
        strcpy (chemfich, getenv ("mailenv"));

        // et accder au fichier contenant le nom du rpertoire pices jointes
        strcpy (chemfich + strlen (chemfich) - 6, ficdir ("FIC_DIRPJ"));
        fic_dirpj = fopen (chemfich, "r");

        // si ce fichier a pu tre ouvert
        if (fic_dirpj)
        {
            // rcuprer le nom du rpertoire destin aux pices jointes
            fgets (chemfich, 80, fic_dirpj);

            // suppression de l'ventuel  \n  en fin de ligne
            if (chemfich [strlen (chemfich) - 1] == '\n')
                chemfich [strlen (chemfich) - 1] = '\0';

            // lecture du fichier termine
            fclose (fic_dirpj);
        }
        // sinon
        else
            // le rpertoire destin aux pices jointes est dfini par dfaut
            strcpy (chemfich + strlen (chemfich) - 5, ficdir ("DIR_PJOINTES"));
    }
    // sinon (rpertoire racine de la messagerie inconnu)
    else
    {
        // saisir au clavier le nom du rpertoire des pices jointes
        // "Nom du rpertoire de destination des pices jointes ?"
        affiche_msg ("REPERT_PJ");
        fgets (chemfich, 80, stdin);

        // suppression de l'ventuel  \n  en fin de ligne
        if (chemfich [strlen (chemfich) - 1] == '\n')
            chemfich [strlen (chemfich) - 1] = '\0';

        // si ce rpertoire n'existe pas
        if (access (chemfich, 0) < 0)
        {
            // demande de confirmation
            // "Rpertoire inexistant, voulez vous le crer ? "
            affiche_msg_nocr ("CREAT_REPERT");
            fgets (reponse, 10, stdin);

            // si rponse ngative
            if (tolower (*reponse) == 'n')
                exit (-1);   // on s'arrtera sans rcuprer les fichiers
        }
    }

    // crer si ncessaire le rpertoire des pices jointes
    mkdir (chemfich, 0755);

    // erreur fatale si le rpertoire n'existe pas encore
    if (access (chemfich, 0) < 0)
        // "Rpertoire %s inexistant"
        errfatale ("REPERT_INEXISTANT", chemfich);
}
