/*
    Fichier supgrosmail.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence-fr.txt

    Traductions des droits d'utilisation dans les fichiers :
    licence-de.txt , licence-en.txt , licence-es.txt ,
    licence-it.txt , licence-nl.txt , licence-pt.txt ,
    licence-eo.txt , licence-eo-utf.txt

    Droits d'utilisation galement sur la page web :
    http://libremail.tuxfamily.org/voir.php?page=droits


    Ce programme dtruit les fichiers mail dpassant la taille
    limite passe en paramtre, sauf si le mail provient d'un
    expditeur autoris.

    La liste des expditeurs autoriss pour envoyer de gros mails
    est mmorise dans un fichier.

    Si un mail est refus, un message d'avertissement est gnr
     l'intention de celui qui l'a expdi, et est envoy en copie
    au destinataire qui ne reoit pas ce message.

    Un fichier de configuration est utilis pour se connecter  la
    boite aux lettres et pour slectionner le serveur SMTP servant
     l'envoi des messages.
*/

#define appli   // pour la dclaration de variables globales  l'application

#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "messages.h"
#include "buflect.h"
#include "ficonf.h"
#include "pop.h"
#include "smtp.h"
#include "testchamp.h"
#include "trtentete.h"
#include "szchemin.h"
#include "mailrep.h"
#include "genchampdate.h"


/* constantes */

// nombre de mails qu'on peut dtruire en une seule passe
#define szmaxliste    100

#define taillemin    2500  // taille max minimale accepte sans confirmation


/* prototypes */
void listergrosmail (int numdepart);
void filtreliste ();
void testfiltre (int posliste);
int  avertir (int posliste);
void ajout_adr (char *dest, char *source);
void transf_ligne (char *message);


/* variables globales au source
   (pour viter des tonnes de passages de paramtres) */

long taillemax;      // taille maximale des mails accepts
char adr_mail [80];  // adresse mail de l'utilisateur
char **listexped;    // mmorise expditeurs autoriss pour gros mails
int  nb_exped;       // nombre d'expditeurs mmoriss dans listexped
int  listegros [szmaxliste];   // numro des gros mails  tester
long taillegros [szmaxliste];  // taille des gros mails  tester
int  szlistegros;    // nombre d'lments dans listegros
char *ficmailrep;    // fichier comptenant un mail pour rponse spcifique

/* buffers rappelant les caractristiques du message */
char bufDate [80], bufFrom [80], bufTo [80], bufCc [80], bufSubject [80];
char adr_exped [80]; // adresse de l'expditeur du message qu'on filtre

/* compteurs */
int  conserves;      // nombre de mails conservs
int  supprimes;      // nombre de mails supprims
int  avertis;        // nombre d'expditeurs de mails supprims avertis
int  non_avertis;    // nombre d'expditeurs de mails supprims non avertis


/* programme principal */

int main (int nbarg, char *varg[])
{
    char serv_smtp [80]; // nom du serveur smtp utilis
    FILE *fconf;         // descripteur du fichier de configuration
    // fichier contenant les adresses d'expditeurs  tester
    char ficadr_OK [szchemin + 11];
    char *car_ficadr_OK; // pointeur sur un caractre de ce fichier


    // rcupration du nom de l'excutable
    memcom (*varg);

    // nombre d'arguments passs en paramtre
    nbarg--;

    // si utilisation d'un mail de rponse spcifique
    if (nbarg == 4 && strcmp (varg [1], "-m") == 0)
    {
        // mmoriser le nom du fichier contenant le mail de rponse
        ficmailrep = varg [2];

        // si ce fichier n'existe pas
        if (access (ficmailrep, 0) != 0)
            errfatale ("MANQUE_FICMAILREP", ficmailrep);

        // sauter les argument traits
        varg = varg + 2;
        nbarg = nbarg - 2;
    }
    // sinon, mail de rponse standard
    else
        ficmailrep = NULL;

    // controle du nombre d'arguments restants
    if (nbarg == 2)
    {
        // rcuprer la taille maximale des mails
        // accepts pour n'importe quel expditeur
        taillemax = atol (varg [1]);

        // erreur de syntaxe si la taille maximale est nulle
        if (! taillemax)
        {
            // "Paramtre taille_max incorrect ou mal plac"
            affiche_err ("ERR_TAILLEMAX");
            return (0);
        }

        // on peut mettre un - devant le paramtre taille
        if (taillemax < 0)
            taillemax = -taillemax;

        // cette taille peut tre exprime en koctets
        if (tolower (varg [1][strlen (varg [1]) - 1]) == 'k')
            taillemax *= 1024;

        // controle de scurit :
        // si cette taille est trs faible, elle doit tre confirme par un !
        if (taillemax < taillemin && varg [1][strlen (varg [1]) - 1] != '!')
        {
            //"Une taille maximale de %ld octets est trs faible pour un mail\n"
            //"Veuillez la confirmer en la faisant suivre d'un !"
            fprintf (stderr, message ("TAILLEMAX_FAIBLE"), taillemax);
            affiche_err ("CONFIRM_TAILLE");

            return (0);
        }

        // ouvrir le fichier de configuration
        fconf = ouvre_ficonf (varg [2]);

        if (fconf)
        {
            // connexion sur le compte mail du serveur pop
            if (connect_pop (fconf))
            {
                // rcuprer la liste des gros mails
                listergrosmail (0);

                // s'il risque d'y avoir des mails  supprimer
                if (szlistegros)
                {
                    // rcuprer le nom du rpertoire racine de la messagerie
                    fgets (ficadr_OK, szchemin, fconf);

                    // fabriquer le chemin d'accs au fichier contenant les
                    // adresses d'expditeurs de mails en copie cache autoriss
                    // <racine>/accept_gros
                    car_ficadr_OK = ficadr_OK + strlen (ficadr_OK);
                    *(car_ficadr_OK - 1) = '/';
                    strcpy (car_ficadr_OK, ficdir ("FIC_ACCEPT_GROS"));

                    // si ce fichier existe
                    if (access (ficadr_OK, 0) == 0)
                        // charger en mmoire la liste de ces expditeurs
                        listexped = charge_valchamp (ficadr_OK, &nb_exped);
                    else
                        // sinon liste vide
                        nb_exped = 0;

                    // recuprer adresse email utilisateur
                    fgets (buf_lect, sizeof (buf_lect), fconf);
                    buf_lect [strlen (buf_lect) - 1] = '\0';

                    // et la mmoriser
                    ajout_adr (adr_mail, buf_lect);

                    // recuprer le nom du serveur smtp
                    fgets (serv_smtp, 80, fconf);
                    serv_smtp [strlen (serv_smtp) - 1] = '\0';

                    // connexion au serveur smtp pour l'envoi des avertissements
                    if (connect_smtp (serv_smtp))
                    {
                        // filtrage des gros mails
                        filtreliste ();

                        // si la liste des gros mails avait atteint sa
                        // taille maximale, mais qu'on a pu en supprimer
                        // on relance une ou plusieurs tape(s) de filtrage
                        while (szlistegros == szmaxliste)
                        {
                            // rcuprer la liste des gros mails restants
                            listergrosmail (listegros [szlistegros - 1]);

                            // s'il en reste
                            if (szlistegros)
                            {
                                // les filtrer
                                // "Nouvelle tape de filtrage"
                                affiche_msg ("NOUV_ETAPE");
                                filtreliste ();
                            }
                        }

                        // fermeture de la connexion smtp
                        deconnect_smtp ();
                    }
                }
                else
                    // "Aucun mail ne dpasse la taille autorise"
                    affiche_msg ("AUCUN_GROSMAIL");

                // se dconnecter proprement du serveur pop
                deconnect_pop ();
            }

            // on n'a plus besoin du fichier de configuration
            fclose (fconf);
        }
    }
    else
      // Syntaxe : %s [-m fichier_mail_rponse] taille_max fichier_configuration
        psyntaxe ("SYNT_SUPGROSMAIL");

    // pour faire plaisir  gcc qui veut une fonction main de type int
    return (0);
}



/* mmorise les numros des mails dpassant la
   taille limite et dont le numro est > numdepart */

void listergrosmail (int numdepart)
{
    long numail;     // numro du mail dont on liste la taille
    long taillemail; // taille de ce mail


    // initialisation
    szlistegros = 0;

    // mmoriser les tailles des mails
    env_pop ("LIST");

    // sauter le message d'entte de la liste
    lire_pop ();

    // lire la taille d'un mail
    lire_pop ();

    // tantque des mails restent  analyser
    while (buf_lect [0] != '.')
    {
        // rcuprer la taille du mail
        sscanf (buf_lect, "%ld%ld", &numail, &taillemail);

        // si ce mail est trop gros, n'a pas t encore mmoris comme mail
        //  supprimer, et que la liste des mails  supprimer n'est pas pleine
        if (taillemail > taillemax && numail > numdepart
                                   && szlistegros < szmaxliste)
        {
            // mmoriser le numro et la taille de ce mail
            listegros  [szlistegros] = numail;
            taillegros [szlistegros++] = taillemail;
        }

        // lire la taille du mail suivant
        lire_pop ();
    }
}



/* supprime les gros mails ne provenant pas d'expditeurs autoriss */

void filtreliste ()
{
    int i;  // compteur


    // initialisation des compteurs de mail traits
    conserves = 0;
    supprimes = 0;
    avertis   = 0;
    non_avertis = 0;

    // test des gros mails et suppression ventuelle
    for (i = 0; i < szlistegros; i++)
        testfiltre (i);

    // dition d'un rcapitulatif
    // "\n%d gros messages conservs, %d supprims\n"
    printf (message ("BILAN_SUPGROSMAIL"), conserves, supprimes);

    if (supprimes)
        // "%d expditeurs avertis, %d non avertis\n"
        printf (message ("BILAN_AVERTIS"), avertis, non_avertis);
}



/* vrifie si un gros mail provient d'un expditeur autoris et si
   ce n'est pas le cas, le dtruit cas en avertissant son expditeur */

void testfiltre (int posliste)
{
    char bufw [80];  // buffer d'envoi d'une requte
    int  numes;      // numro du mail trait


    // initialisation
    bufDate [0]    = '\0';
    bufFrom [0]    = '\0';
    bufTo [0]      = '\0';
    bufCc [0]      = '\0';
    bufSubject [0] = '\0';
    numes = listegros [posliste];

    // message de suivi de droulement
    // "\rControle expditeur du message : %d"
    printf (message ("TEST_EXPED"), numes);
    fflush (stdout);

    /* demande de lecture de l'entte du message */
    sprintf (bufw, "TOP %d 1", numes);
    env_pop (bufw);

    // lecture et mmorisation des caractristiques du message
    do
    {
        // lire une ligne de l'entte du message
        lire_pop ();

        // convertir les caractres spciaux
        majlignentete ();

        // mmorisation des caractristiques importantes
        // pour cette fonction, on n'a besoin que de la ligne From:
        // mais en mmorisant le reste, on vite de relire l'entte
        // du mail dans la fonction avertir
        if (start ("Date"))
            membuf (bufDate);
        else if (start ("From"))
            membuf (bufFrom);
        else if (start ("To"))
            membuf (bufTo);
        else if (start ("Cc"))
            membuf (bufCc);
        else if (start ("Subject"))
            membuf (bufSubject);
    }
    // lecture termine si ligne limite  un .
    while (buf_lect [0] != '.' || buf_lect [1] != '\0');

    // extraire l'adresse de l'expditeur
    ajout_adr (adr_exped, bufFrom + 5);

    // si le mail ne provient pas d'un expditeur autoris
    if ((! *adr_exped) || (! trouve_valchamp (adr_exped, listexped, nb_exped)))
    {
        // envoyer un mail  l'expditeur du message
        if (bufFrom [0])
            if (avertir (posliste))
                avertis ++;
            else
                non_avertis ++;

        // demande de destruction du message
        sprintf (bufw, "DELE %d", numes);
        env_pop (bufw);
        lire_pop ();
        supprimes ++;
    }
    else
        conserves ++;
}



/* avertit l'expditeur d'un gros mail que son message est refus */

int avertir (int posliste)
{
    char bufw [120];  // buffer d'envoi d'une ligne ou d'une requte smtp


    // l'envoi du mail d'avertissement suppose l'accs  un fichier de donnes
    if (! acces_mailrep (ficmailrep, "supgros"))
        return (0);

    // expditeur du message
    sprintf (bufw, "MAIL FROM: <%s>", adr_mail);
    envoie_smtp (bufw);
    lire_smtp ();

    // destinataire principal du message
    sprintf (bufw, "RCPT TO: <%s>", adr_exped);
    envoie_smtp (bufw);
    lire_smtp ();

    // si destinataire refus, on le signale
    if (buf_lect [0] != '2')
    {
        // Affichage de l'adresse  problme
        // "Destinataire %s refus\n"
        printf (message ("REFUS_DEST"), adr_exped);
        puts (buf_lect);

        // et on n'envoie aucun mail d'avertissement
        envoie_smtp ("RSET");
        lire_smtp ();
        return 0;  // mail non envoy
    }

    // l'expditeur du message le recevra en copie
    sprintf (bufw, "RCPT TO: <%s>", adr_mail);
    envoie_smtp (bufw);
    lire_smtp ();

    // prparation de l'envoi du message
    envoie_smtp ("DATA");
    lire_smtp ();

    // gnration de l'entte du message
    gen_champ_date (bufw);
    envoie_smtp (bufw);
    sprintf (bufw, "From: <%s>", adr_mail);
    envoie_smtp (bufw);
    sprintf (bufw, "To%s", bufFrom + 4);
    envoie_smtp (bufw);
    sprintf (bufw, "Cc: <%s>", adr_mail);
    envoie_smtp (bufw);

    // "Subject: Votre message du ..."
    sprintf (bufw, "Subject: %s %s", chaine_mailrep ("SUJET"), bufDate + 4);
    envoie_smtp (bufw);

    // "Content-Type: text/plain; charset=..."
    sprintf (bufw, "Content-Type: text/plain; charset=%s",
                                  chaine_mailrep ("CHARSET"));
    envoie_smtp (bufw);

    envoie_smtp ("Content-Transfer-Encoding: 8bit");

    // filtre supgrosmail : logiciel libre multilingue"
    sprintf (bufw, "User-Agent: filtre %s : %s\n", nomcom (),
                                message ("QUALIF_LIBREMAIL"));
    envoie_smtp (bufw);
    envoie_smtp ("");

    // "Vous m'avez envoy un mail ayant les caractristiques suivantes :"
    envoie_smtp (chaine_mailrep ("ANNONCE"));
    envoie_smtp ("");

    // rappel du message concern
    transf_ligne (bufDate);
    transf_ligne (bufFrom);
    transf_ligne (bufTo);
    transf_ligne (bufCc);
    transf_ligne (bufSubject);

    // Taille: %ld octets
    sprintf (bufw, chaine_mailrep ("TAILLE"), taillegros [posliste]);
    transf_ligne (bufw);
    envoie_smtp ("");

    // envoi du message explicatif
    envoi_mailrep ();

    // indicateur de fin du message
    envoie_smtp (".");
    lire_smtp ();

    return 1;   // bon droulement
}



/* rcupre l'adresse mail entre les < > s'ils
   existent et la rajoute au buffer destination */

void ajout_adr (char *dest, char *source)
{
    int orig, debut, posdest;

    orig = 0;

    // chercher le @
    while (source [orig] != '@' && source [orig])
        orig ++;

    // remonter au debut de l'adresse
    while (orig > 0 && source [orig] != ' ' && source [orig] != '<')
        orig--;

    // l'adresse commence sur un caractre significatif
    if (source [orig] == '<' || source [orig] == ' ')
        debut = orig + 1;
    else
        debut = orig;

    // copie de l'adresse
    posdest = 0;

    // on s'arrte si l'on rencontre un  > , un blanc ou une fin de chaine
    while (source [debut] != '>' && source [debut] & 0xDF)
        // l'adresse est mmorise en minuscules
        dest [posdest++] = tolower (source [debut++]);

    // terminer l'expression de l'adresse
    dest [posdest] = '\0';
}



/* copie dans le mail  envoyer d'une ligne du mail reu */

void transf_ligne (char *message)
{
    char bufw [82];  // buffer contenant la ligne  envoyer

    if (*message)
    {
        strcpy (bufw, "> ");
        strcpy (bufw + 2, message);
        envoie_smtp (bufw);
    }
}
