/*
    Fichier testchamp.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence-fr.txt

    Traductions des droits d'utilisation dans les fichiers :
    licence-de.txt , licence-en.txt , licence-es.txt ,
    licence-it.txt , licence-nl.txt , licence-pt.txt ,
    licence-eo.txt , licence-eo-utf.txt

    Droits d'utilisation galement sur la page web :
    http://libremail.tuxfamily.org/voir.php?page=droits


    Bibliothque de fonctions permettant de rcuprer dans un fichier
    la liste des valeurs interdites dans un champ d'un mail puis de
    vrifier si un champ de mail contient l'une de ces valeurs.
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "buflect.h"
#include "messages.h"
#include "testchamp.h"



/* charge en mmoire le fichier contenant une liste de chaines de caractres */

char ** charge_valchamp (char *nomfic, int *sz_liste)
{
    char valchaine [80]; // une chaine de ce fichier
    char *nouvchaine;    // chaine retourne alloue dynamiquement
    FILE *fliste;        // descripteur du fichier des chaines  mmoriser
    char **liste;        // liste des chaines de caractres issues du fichier
    int  pos_liste;      // position dans la liste de chaines mmorises


    // initialisation : pas encore de chaine mmorise
    *sz_liste = 0;

    // accs au fichier contenant les chaines  mmoriser
    fliste = fopen (nomfic, "r");

    // si ce fichier existe
    if (fliste)
    {
        // compter le nombre de chaines de caractres qu'il contient
        while (fgets (valchaine, sizeof (valchaine), fliste))
            *sz_liste = *sz_liste + 1;

        // allouer la zone mmoire pour mmoriser la liste des chaines
        // la taille de la liste alloue est optimale et non limite
        //  la compilation
        liste = malloc (*sz_liste * sizeof (char *));

        if (! liste)
            // "Manque de place mmoire, l'application ne peut fonctionner"
            errfatale ("MANQUE_MEMOIRE", NULL);

        // on revient au dbut du fichier
        rewind (fliste);

        // initialisation
        pos_liste = 0;

        // rcuprer et mmoriser les diffrentes chaines de caractres
        while (fgets (valchaine, sizeof (valchaine), fliste))
        {
            // supprimer le \n en fin de ligne
            valchaine [strlen (valchaine) - 1] = '\0';

            // allouer une variable pour mmoriser la chaine lue
            nouvchaine = (char *) malloc (strlen (valchaine) + 1);

            if (! nouvchaine)
                // "Problme d'allocation mmoire"
                errfatale ("MANQUE_MEMOIRE", NULL);

            // recopier la chaine
            strcpy (nouvchaine, valchaine);

            // mmoriser la chaine dans la liste (on copie l'adresse)
            liste [pos_liste ++] = nouvchaine;
        }

        // termin avec ce fichier
        fclose (fliste);
    }
    else
    {
        // message d'erreur si fichier des chaines  mmoriser manquant
        // "Pas de fichier %s aucun filtrage possible"
        aff_err_arg ("FIC_FILTR_ABSENT", nomfic);
    }

    // retourner le nombre d'lments de la liste rcuprs
    return (liste);
}



/* recherche si le champ du mail passe en paramtre
   contient l'une des chaines de la liste */

int trouve_valchamp (char *valchamp, char **liste, int sz_liste)
{
    char *chainetest;  // chaine de la liste  tester
    int  i, j, k;      // compteurs

    // on compare la chaine valchamp avec toutes les chaines de la liste
    for (i = 0; i < sz_liste; i++)
    {
        // permettra d'viter des calculs de dplacements dans tableau
        chainetest = liste [i];

        // initialisation
        k = 0;

        // vrifier si la chaine chainetest se trouve dans valchamp
        do
        {
            j = 0;

            while (valchamp [j+k] == chainetest [j])
                // termin si tous les caractres de chainetest trouvs
                if (chainetest [++j] == 0)
                    return 1;
        }
        // sinon recherche dans la suite de valchamp
        while (valchamp [k++]);
    }

    // aucune des chaines de la liste n'a t trouve dans valchamp
    return (0);
}
