/*
    Fichier vmaildir.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence-fr.txt

    Traductions des droits d'utilisation dans les fichiers :
    licence-de.txt , licence-en.txt , licence-es.txt ,
    licence-it.txt , licence-nl.txt , licence-pt.txt ,
    licence-eo.txt , licence-eo-utf.txt

    Droits d'utilisation galement sur la page web :
    http://libremail.tuxfamily.org/voir.php?page=droits


    Ce programme affiche l'arborescence du rpertoire pass
    en paramtre, rpertoire courant par dfaut.

    Il est recommand de lancer cet outils depuis (ou en
    passant en paramtre) le rpertoire racine des fichiers
    mails tel qu'il est dfini dans le fichier de configuration
    xxxx.cfg

    Il est possible de slectionner l'un des rpertoires
    affichs pour consulter les fichiers mails qu'il contient
    et ventuellement rpondre, transfrer, ou dtruire le message.

    Pour ces oprations, cet outil appelle vmailsj
*/

#define appli   // pour la dclaration de variables globales  l'application

#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <dirent.h>
#include <sys/stat.h>
#include <sys/types.h>
#include "messages.h"
#include "modepage.h"
#include "carspe.h"
#include "szchemin.h"


/*
   Par dfaut ce programme dtecte les noms
   de fichiers mails issus de libremail.

   Dcommenter la ligne qui suit pour accepter
   aussi ceux issus de microsoft internet mail.
*/
// #define compat_msoft

/*
   Les noms du premier mail envoy et du premier mail
   reu avec microsoft internet mail sont respectivement
   EAAAAAAA et RAAAAAAA.N (RAAAAAAA.R aprs lecture).

   Ensuite, les mails sont numrots en changeant d'abord
   les caractres de droites (36 caractres possibles).

   On peut ainsi numroter 1296 mails sur 2 caractres,
   46656 mails sur 3 caractres, plus de 1,6 millions
   de mails sur 4 caractres, etc...

   La constante nb_a permet de prciser le nombre minimal
   de A que l'on veut avoir cot gauche pour qu'un nom de
   fichier soit considr comme celui d'un fichier mail.
*/
#ifdef  compat_msoft
#define nb_a              3  // de 0  7
#endif


#define max_mdir_ini    100  // nombre max initial de rpertoires de mails
#define mdir_accept   50000  // nombre max de rpertoires qu'on accepte
#define lg_nomdir        32  // longueur max d'un nom de rpertoire


/* type de donnes */
typedef char nomdir [lg_nomdir];


/* prototypes */
void exploredir (char *repertoire);
int  ajoutdirpossible ();
int  fichier_mail (char *nomfic);
nomdir *maj_espace (nomdir * oldliste, int * taille);
void trinoms (nomdir *liste, int fin);
void navigation ();
void detectemail (int numrep);
void gen_format_cpt ();
void affligne (int numligne);


/* variables globales au source
   (pour viter des tonnes de passages de paramtres) */

char **maildir;       // liste des rpertoires de mails
int  *nb_mails;       // nombre de mails pour chaque rpertoire
int  *nb_nonlus;      // nombre de mails non lus pour chaque rpertoire
int  max_maildir;     // nombre max d'lments dans maildir, nb_mails, nb_nonlus
int  numsortie = 0;   // position du rpertoire des mails  envoyer
int  numpoub   = 0;   // position du rpertoire des mails dtruits
int  deborde_mem = 0; // mmorise un dbordement mmoire
int  opt_inv;         // option d'inversion de l'ordre d'affichage pour vmailsj
int  opt_cn;          // options d'affichage du nombre de mails non lus et total
char formatcpt [10];  // format d'affichage du nombre de mails non lus et total
int  coldebnomrep;    // colonne du dbut d'afffichage d'un nom de rpertoire

// ces deux variables permettent d'viter des appels rptitifs de ficdir ()
char dir_sortie [szmax_ficdir]; // sous rpertoire des mails  envoyer
char dir_poub   [szmax_ficdir]; // sous rpertoire des mails avant destruction


/* programme principal */

int main (int nbarg, char *varg[])
{
    char nomdir [szchemin];
    char *ajout;


    // rcupration du nom de l'excutable
    memcom (*varg);

    // se positionner sur le premier argument de vmaildir
    varg ++;
    nbarg --;

    // initialisations
    nb_lignes = 0;
    opt_inv   = 0;
    opt_cn    = 0;

    // rcupration des options de fonctionnement ventuelles
    while (nbarg > 0 && **varg == '-' && varg [0][2] == '\0')
    {
        switch (varg [0][1])
        {
            // option inversion de l'ordre d'affichage (pour vmailsj)
            case 'i' :
            case 'r' : opt_inv = 1;
                       varg ++;
                       nbarg --;
                       break;

            // option d'affichage du nombre de mails non lus
            case 'n' : if (! opt_cn)  // prise en compte si option -c absente
                           opt_cn = 1;
                       varg ++;
                       nbarg --;
                       break;

            // option d'affichage des nombres de mails non lus et total
            case 'c' : opt_cn = 2;   // prioritaire sur l'option -n
                       varg ++;
                       nbarg --;
                       break;

            default  : // "Option %s incorrecte"
                       aff_err_arg ("ERR_OPTION", *varg);

                       // "Syntaxe : %s [-(c|n)] [-i] [rpertoire]"
                       psyntaxe ("SYNT_VMAILDIR");
        }
    }

    // rcupration ventuelle du nom du rpertoire racine des mails
    if (nbarg == 1)
    {
        // se positionner dans ce rpertoire
        if (chdir (*varg) < 0)
        {
            // "Rpertoire %s inexistant"
            aff_err_arg ("REPERT_INEXISTANT", *varg);
            return (0);
        }
    }

    // rcuprer et mmoriser le nom de 2 sous rpertoires importants
    strcpy (dir_sortie, ficdir ("DIR_SORTIE"));
    strcpy (dir_poub, ficdir ("DIR_POUBELLE"));

    // rcuprer le chemin d'accs absolu de la racine des mails
    getcwd (nomdir, szchemin);

    // dterminer le nom du rpertoire d'envoi des mails
    ajout = nomdir + strlen (nomdir);
    *ajout = '/';
    strcpy (ajout + 1, dir_sortie);

    // et le mmoriser dans une variable d'environnement
    setenv ("mailenv", nomdir, 1);

    // faire de mme avec le rpertoire poubelle des fichiers mail
    strcpy (ajout + 1, dir_poub);
    setenv ("mailpoub", nomdir, 1);

    // fixer la taille initiale de la liste des rpertoires
    max_maildir = max_mdir_ini;

    // allouer la liste des rpertoires et des compteurs de mails
    // (on commence par les tableaux les moins encombrants)
    nb_mails   = malloc (max_maildir * sizeof (int));
    nb_nonlus  = malloc (max_maildir * sizeof (int));
    maildir    = malloc (max_maildir * sizeof (char *));

    // vrification allocation
    if (! maildir)
        // "Manque de place mmoire, le logiciel %s ne peut fonctionner"
        errfatale ("MANQUE_MEMOIRE", nomcom ());

    // mmoriser l'arborescence du rpertoire racine des mails
    exploredir (".");

    // on gnre un message en fin de liste si dbordement mmoire
    if (deborde_mem || max_maildir > mdir_accept)
    {
        if (nb_lignes + 2 >= max_maildir)
            nb_lignes = nb_lignes - 2;

        maildir [nb_lignes] = "";
        nb_mails [nb_lignes++] = -1;

        // "Trop de sous rpertoires, liste tronque"
        strcpy (nomdir, message ("DEBORD_SOUS_REP"));
        maildir [nb_lignes] = nomdir;
        nb_mails [nb_lignes] = -1;
        nb_nonlus [nb_lignes++] = 1;   // pour la surbrillance
    }

    // si demande d'affichage du nombre de mails (non lus ou/et total)
    if (opt_cn)
        // prparer le format d'affichage de ces valeurs
        gen_format_cpt ();

    // vrification de l'accs  ce message avant le passage en mode raw
    message ("RACINE_MAILS");

    // afficher les rpertoire et naviguer dans la liste
    navigation ();

    // pour faire plaisir  gcc qui veut une fonction main de type int
    return (0);
}



/* exploration du rpertoire pass en paramtre */

void exploredir (char *repertoire)
{
    DIR *repert;
    struct dirent *entree;
    struct stat   descfic;
    char   nomfic [szchemin]; // chemin d'accs  un fichier
    nomdir *listedir;    // liste des sous rpertoires trouvs
    int    nbdir_alloc;  // nombre de sous repertoires allous
    int    nbdir;        // nombre de sous repertoires mmoriss
    int    sznomrep;     // longueur du nom de rpertoire qu'on analyse
    int    i;            // simple compteur


    // si trop de rpertoires dj explors, on sort
    if (max_maildir > mdir_accept)
        return;

    // accder au rpertoire
    repert = opendir (repertoire);

    // sortie sur erreur si l'on n'a pas pu
    if (repert == 0)
        return;

    // mmorisation longueur du nom de rpertoire
    sznomrep = strlen (repertoire);

    // mmoriser le chemin d'accs au rpertoire
    if (nb_lignes == 0)
    {
        // cas particulier de rpertoire racine
        maildir [nb_lignes] = ".";
    }
    // sinon si on peut mmoriser d'autres lments
    else if (ajoutdirpossible ())
    {
        // cas gnral : on ne mmorise pas le ./ des noms de sous rpertoires
        maildir [nb_lignes] = (char *) malloc (strlen (repertoire) - 1);

        if (maildir [nb_lignes])
            strcpy (maildir [nb_lignes], repertoire + 2);
        else
            // une erreur mmoire  ce niveau n'est pas facilement rcuprable
            // "Manque de place mmoire, le logiciel %s ne peut fonctionner"
            errfatale ("MANQUE_MEMOIRE", nomcom ());

        // mmoriser la position des rpertoires sortie et poubelle
        if (strcmp (repertoire + 2, dir_sortie) == 0)
            numsortie = nb_lignes;
        else if (strcmp (repertoire + 2, dir_poub) == 0)
            numpoub = nb_lignes;
    }
    // sinon mmoriser un dbordement mmoire
    else
    {
        deborde_mem = 1;
        return;
    }

    // initialisation : pas encore de mail trouv dans ce rpertoire
    nb_mails  [nb_lignes] = 0;
    nb_nonlus [nb_lignes] = 0;

    // rcuprer le nombre de liens du rpertoire
    stat (repertoire, &descfic);

    // et allouer le bon nombre d'entres dans la liste des
    // sous rpertoires avec une scurit de 2 (correspondant
    //  . et ..) pour viter erreur de segmentation
    nbdir_alloc = descfic.st_nlink;
    listedir = (nomdir *) calloc (nbdir_alloc, lg_nomdir);

    if (! listedir)
    {
        deborde_mem = 1;
        return;
    }

    // aucun sous rpertoire encore mmoris
    nbdir = 0;

    // lire un nom de fichier ou de rpertoire
    entree = readdir (repert);

    // tant qu'il y a des noms de fichier ou de rpertoires  lire
    while (entree)
    {
        // si le nom ne commence pas par un .
        // (vitera de boucler sur les rpertoires . et ..)
        // et qu'il n'est pas trop long
        if (entree->d_name [0] != '.' &&
           (sznomrep + strlen (entree->d_name) < szchemin))
        {
            // rcuprer les caractristiques du fichier courant
            sprintf (nomfic, "%s/%s", repertoire, entree->d_name);
            stat (nomfic, &descfic);

            // on ne mmorisera que les noms des rpertoires
            if ((descfic.st_mode & S_IFMT) == S_IFDIR)
            {
                // tronquer les noms  32 caractres
                if (strlen (entree->d_name) >= lg_nomdir)
                    entree->d_name [lg_nomdir - 1] = '\0';

                // prcaution pour viter un dbordement de tableau
                if (nbdir >= nbdir_alloc)
                    listedir = maj_espace (listedir, &nbdir_alloc);

                // mmoriser le nom du sous rpertoire
                strcpy (listedir [nbdir ++], entree->d_name);
            }
            // et la prsence de fichiers mail dj lus ou en attente de lecture
            else if (fichier_mail (entree->d_name))
            {
                nb_mails [nb_lignes] ++;

                // test si mail en attente de lecture
                if (entree->d_name [8] == '.' &&
                                      tolower (entree->d_name [9]) == 'n')
                {
                    nb_nonlus [nb_lignes] ++;
                }
            }
        }

        // et lire le nom du fichier suivant
        entree = readdir (repert);
    }

    // librer le rpertoire
    closedir (repert);

    // mettre  jour le compteur
    nb_lignes ++;

    // tri alphabtique des noms de sous rpertoires
    trinoms (listedir, nbdir - 1);

    // exploration des sous rpertoires trouvs
    for (i = 0; i < nbdir; i++)
    {
        sprintf (nomfic, "%s/%s", repertoire, listedir [i]);
        exploredir (nomfic);
    }

    // libration de l'espace mmoire du tableau listedir
    free (listedir);
}



/* vrifie si l'on peut insrer un lment de plus dans le tableau
   maildir, et redimmensionne ce tableau si ncessaire */

int ajoutdirpossible ()
{
    char **nouvmaildir;  // adresse du tableau maildir de remplacement
    int  *nouv_nbmails;  // adresse du tableau nb_mails de remplacement
    int  *nouv_nbnonlus; // adresse du tableau nb_nonlus de remplacement
    int  nouvtaille;     // et sa taille
    int  element;        // compteur : numro d'lment dans les tableaux


    // cas simple : il reste au moins une place de libre dans le tableau
    if (nb_lignes + 1 < max_maildir)
        return (1);

    // calculer la nouvelle taille des tableaux
    // l'augmentation est alternativement de 50 % ou 33 % de manire
    //  ce que la taille double aprs 2 rallocations
    if (max_maildir % 3)
        nouvtaille = max_maildir + (max_maildir / 2);
    else
        nouvtaille = max_maildir + (max_maildir / 3);

    // allocation mmoire des nouveaux tableaux
    nouv_nbmails  = malloc (nouvtaille * sizeof (int));
    nouv_nbnonlus = malloc (nouvtaille * sizeof (int));
    nouvmaildir   = malloc (nouvtaille * sizeof (char *));

    // vrification allocation
    if (nouvmaildir)
    {
        // copie du contenu des ancientableau dans le nouveaux
        for (element = 0; element < max_maildir; element ++)
        {
            nouvmaildir   [element] = maildir   [element];
            nouv_nbmails  [element] = nb_mails  [element];
            nouv_nbnonlus [element] = nb_nonlus [element];
        }

        // destruction des anciens tableaux
        free (maildir);
        free (nb_mails);
        free (nb_nonlus);

        // que l'on remplace par les nouveaux
        maildir   = nouvmaildir;
        nb_mails  = nouv_nbmails;
        nb_nonlus = nouv_nbnonlus;
        max_maildir = nouvtaille;
    }

    // retourne le rsultat de la possibilit d'insertion d'lments
    return (nb_lignes + 1 < max_maildir);
}



/* dtermine si un nom de fichier pass en paramtre
   correspond  un nom de fichier mail */

int fichier_mail (char *nomfic)
{
    char car; // pour viter de trop manipuler des tableaux
    int  i;   // compteur


    // test du premier caractre du nom de fichier
    car = tolower (nomfic [0]);

    // si le nom de fichier commence par l'une des bonnes lettres possibles
    if (car == 'r' || car == 'e'
#ifdef compat_msoft
                   || car == 's'
#endif
       )
        // on va examiner les caractres qui suivent
        i = 1;
    else
        // sinon ce n'est pas un fichier mail
        return 0;

    // test des caractres qui suivent
    while (i < 8)
    {
#ifdef compat_msoft
        car = tolower (nomfic [i]);
#else
        car = nomfic [i];
#endif

        // si le ime caractre est bon
        if (('0' <= car && car <= '9')
#ifdef compat_msoft
         || (car == 'a')
         || (i >= nb_a && 'b' <= car && car <= 'z')
#endif
           )
            // examiner le caractre suivant
            i++;
        else
            // sinon ce n'est pas un fichier mail
            return 0;
    }

    // le si nom de fichier fait 8 caractres
    if (nomfic [8] == '\0')
        // il s'agit d'un nom de fichier mail
        return 1;

    // sinon si le 9me caractre est un . et qu'il y a 10 caractres en tout
    else if (nomfic [8] == '.' && nomfic [10] == '\0')
    {
        // examiner le dernier caractre
        car = tolower (nomfic [9]);

        if (car == 'n')
            return 1;    // fichier mail reu et non encore lu

#ifdef compat_msoft
        else if (car == 'r')
            return 1;   // fichier micro$oft internet mail dj lu
#endif
        else
            return 0;   // ce n'est pas un fichier mail
    }
    else
        // sinon ce n'est pas un fichier mail
        return 0;
}



/* augmentation de la taille d'une liste de sous rpertoires */

nomdir * maj_espace (nomdir * oldliste, int * taille)
{
    nomdir *newliste;    // nouveau tableau


    // allocation d'un tableau plus grand
    newliste = (nomdir *) calloc (*taille + 4, lg_nomdir);

    // controle
    if (! newliste)
        // "Manque de place mmoire, l'application ne peut fonctionner"
        errfatale ("MANQUE_MEMOIRE", NULL);

    // copie de l'ancien tableau
    memcpy (newliste, oldliste, *taille * lg_nomdir);

    // libration de la mmoire
    free (oldliste);

    // retour de la nouvelle adresse et taille du tableau
    *taille = *taille + 4;
    return (newliste);
}



/* tri alphabtique des noms de sous rpertoires */

void trinoms (nomdir *liste, int fin)
{
    char tampon [lg_nomdir];  // chaine de caractres pour permutation
    int  i;   // indice dans la liste


    // initialisation
    i = 0;

    while (i < fin)
    {
        // on compare 2 lments conscutifs
        if (strcmp (liste [i], liste [i+1]) > 0)
        {
            // interversion de 2 noms
            strcpy (tampon, liste [i]);
            strcpy (liste [i], liste [i+1]);
            strcpy (liste [i+1], tampon);

            // ce qui peut supposer de comparer un nom dplac avec le prcdent
            if (i > 0)
                i--;
            else
                i++;
        }
        else
            i++;    // on progresse dans la liste
    }
}


/* affiche la liste des sous rpertoire et permet de
   la parcourir mme si elle tient sur plusieurs pages */

void navigation ()
{
    int car;     // caractre tap au clavier
    int erreurs; // nombre frappes caractres inconnus comme commandes


    // configurer la liaison clavier pour lecture directe avec timeout
    mode_raw ();

    // initialisation
    lignecran = 1;
    lignecour = 0;
    erreurs = 0;
    affpage ();

    do
    {
        // lire et traiter une touche du clavier
        car = leccar ();

        switch (car)
        {                  // dplacement dans la liste des rpertoires
            case MONTE   : monte (1);
                           erreurs = 0;
                           break;

            case DESCEND : descend (1);
                           erreurs = 0;
                           break;

            case PAGEUP  : monte (lignepage + lignecran - 2);
                           erreurs = 0;
                           break;

            case PAGEDOWN: descend (2 * lignepage - lignecran - 2);
                           erreurs = 0;
                           break;

            case HOME    :
            case HOMEg   : monte (lignecour);
                           erreurs = 0;
                           break;

            case FIN     :
            case FINg    : descend (nb_lignes - lignecour - 1);
                           erreurs = 0;
                           break;

            case 0       : // aide si trop d'erreurs ou  la demande
            case F1      : effpage ();
                           // "Touches utilisables :\n"
                           // "flches Pageup, Pagedown, Home et Fin"
                           // "pour se dplacer d'une ou plusieurs lignes"
                           // "v, l ou entre pour Voir (Lire)"
                           // "les mails du rpertoire"
                           // "n ou Inser pour en crer un Nouveau"
                           // "Control L pour rafficher la page"
                           // "Esc pour sortir de ce programme\n"
                           // "Appuyer sur une touche pour continuer"
                           affiche_msg ("AIDE_CHOIX_ADR-1");
                           affiche_msg ("AIDE_CHOIX_ADR-2");
                           affiche_msg ("AIDE_CHOIX_ADR-3");
                           affiche_msg ("AIDE_VMAILDIR-1");
                           affiche_msg ("AIDE_VMAILDIR-2");
                           affiche_msg ("AIDE_VMAILDIR_SJ");
                           affiche_msg ("AIDE_CHOIX_ADR-7");
                           affiche_msg ("AIDE_VMAIL");
                           affiche_msg ("ATTENTE_CLAVIER");
                           leccar ();
                           // pas de break, on raffiche la page

            case CTRL    : affpage ();
                           erreurs = 0;
                           break;

            case 'l'     :
            case 'v'     : // ici, on regarde les descriptifs des mails
            case '\n'    : if (opt_inv)
                               execom ("vmailsj -i", maildir [lignecour]);
                           else
                               execom ("vmailsj", maildir [lignecour]);

                           // maj du mode d'affichage des rpertoires
                           detectemail (lignecour);
                           detectemail (numsortie);
                           detectemail (numpoub);

                           // rafficher la page d'cran
                           affpage ();
                           erreurs = 0;
                           break;

            case INSERT  : // cration d'un nouveau mail
            case 'n'     : execom ("cremail", " ");

                           // maj du mode affichage rpertoire mails  envoyer
                           detectemail (numsortie);

                           // rafficher la page d'cran
                           affpage ();
                           erreurs = 0;
            case ESC     : break;

                           // sortie aussi sur :q
            case ':'     : car = leccar ();

                           if (car == 'q')
                           {
                               car = ESC;
                               break;
                           }

            default      : putchar (7);  // bip

                           // si trop d'erreurs, afficher l'aide
                           if (++erreurs == 5)
                               ungetc (0, stdin);  // on ira sur l'aide
        }
    }
    while (car != ESC);

    // descendre en bas de page
    while (lignecran++ < lignepage && lignecour++ < nb_lignes)
        putchar ('\n');

    // retour  la configuration standard du clavier
    mode_normal ();
}



/* recherche de fichiers mail dans le rpertoire pass en paramtre */

void detectemail (int numrep)
{
    DIR *repert;
    struct dirent *entree;
    struct stat   descfic;
    char   nomfic [szchemin]; // chemin d'accs  un fichier


    // accder au rpertoire
    repert = opendir (maildir [numrep]);

    // sortie sur erreur si l'on n'a pas pu
    if (repert == 0)
        return;

    // initialisation : pas encore de mail trouv dans ce rpertoire
    nb_mails  [numrep] = 0;
    nb_nonlus [numrep] = 0;

    // lire un nom de fichier
    entree = readdir (repert);

    // tant qu'il y a des noms de fichier  lire
    while (entree)
    {
        // rcuprer les caractristiques du fichier courant
        sprintf (nomfic, "%s/%s", maildir [numrep], entree->d_name);
        stat (nomfic, &descfic);

        // si c'est un fichier mail
        if ((descfic.st_mode & S_IFMT) == S_IFREG
                                       && fichier_mail (entree->d_name))
        {
            nb_mails [numrep] ++;

            // test si mail en attente de lecture
            if (entree->d_name [8] == '.' &&
                                  tolower (entree->d_name [9]) == 'n')
            {
                nb_nonlus [numrep] ++;
            }
        }

        // et lire le nom du fichier suivant
        entree = readdir (repert);
    }

    // librer le rpertoire
    closedir (repert);
}



/* gnre le format d'affichage du nombre de mails non lus
   et ventuellement du nombre total de mails */

void gen_format_cpt ()
{
    int max_nbmails = 0;
    int max_nonlus  = 0;
    int puis10;
    int col_nl, col_m;
    int i;


    // chercher le plus grand nombre de mails non lus
    // parmi tous les rpertoires mmoriss
    for (i = 0; i < nb_lignes; i++)
        if (nb_nonlus [i] > max_nonlus)
            max_nonlus = nb_nonlus [i];

    // calculer le nombre de chiffres pour afficher ce nombre
    puis10 = 10;
    col_nl  = 1;

    while (max_nonlus >= puis10)
    {
        puis10 = puis10 * 10;
        col_nl ++;
    }

    // si on doit afficher aussi le nombre total de mails
    if (opt_cn > 1)
    {
        // chercher le plus grand nombre de mails
        // parmi tous les rpertoires mmoriss
        for (i = 0; i < nb_lignes; i++)
            if (nb_mails [i] > max_nbmails)
                max_nbmails = nb_mails [i];

        // calculer le nombre de chiffres pour afficher ce nombre
        puis10 = 10;
        col_m  = 1;

        while (max_nbmails >= puis10)
        {
            puis10 = puis10 * 10;
            col_m ++;
        }

        // gnrer le format d'affichage des 2 nombres
        sprintf (formatcpt, "%%%dd/%%-%dd ", col_nl, col_m);

        // et mmoriser la largeur de cet affichage
        coldebnomrep = col_nl + col_m + 2;
    }
    // sinon
    else
    {
        // gnrer le format d'affichage du nombre de mails non lus
        sprintf (formatcpt, "%%%dd ", col_nl);

        // et mmoriser la largeur de cet affichage
        coldebnomrep = col_nl + 1;
    }
}



/* affiche le nom de rpertoire pass en
   paramtre sans dborder de la fentre */

void affligne (int numlig)
{
    int  i;
    char *carcourant;


    // positionnement au dbut du nom de rpertoire
    if (strcmp (maildir [numlig], ".") == 0)
        // cas particulier du rpertoire racine
        // "<racine>"
        carcourant = message ("RACINE_MAILS");
    else
        // cas gnral
        carcourant = maildir [numlig];

    // afficher en bleu fonc les rpertoires sans mail
    if (!nb_mails [numlig])
        sombre ();
    // et en surbrillance ceux qui ont des mails non lus
    else if (nb_nonlus [numlig])
        clair ();

    // si demand, afficher les nombres de mails
    if (opt_cn)
        printf (formatcpt, nb_nonlus [numlig], nb_mails [numlig]);

    // initialisation numro de colonne d'affichage
    i = coldebnomrep;

    // on arrte l'affichage en fin de chaine ou en bord de fentre
    while (*carcourant  && i++ < colonpage)
    {
        putchar (*carcourant);
        carcourant ++;
    }

    // retour  l'affichage en luminosit normale
    lumnor ();
}
