/*
    Fichier vmailsj.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence-fr.txt

    Traductions des droits d'utilisation dans les fichiers :
    licence-de.txt , licence-en.txt , licence-es.txt ,
    licence-it.txt , licence-nl.txt , licence-pt.txt ,
    licence-eo.txt , licence-eo-utf.txt

    Droits d'utilisation galement sur la page web :
    http://libremail.tuxfamily.org/voir.php?page=droits


    Ce programme affiche les caractristique de chaque mail
    mmoris dans un fichier.

    Pour chaque fichier mail, il y a une ligne d'affichage de la forme :
     expditeur      sujet     date et heure d'envoi
     destinataire    sujet     date et heure d'envoi
    selon qu'il s'agisse d'un mail reu ou envoy (ou  envoyer)

    On peut naviguer dans la liste avec les flches pour slectionner
    le mail  visualiser.
    Une fois slectionn, on pourra rpondre  ce mail ou le transfrer.

    Par dfaut, on analyse les mails du rpertoire courant.
    On peut aussi indiquer un autre rpertoire de recherche.

    Cet outil peut tre lanc automatiquement par la commande
    vmaildir lorsqu'on slectionne un rpertoire de l'arborescence
    des mails.
    Il lance  son tour vmailfic pour la visalisation des mails.
*/

#define appli   // pour la dclaration de variables globales  l'application

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <dirent.h>
#include <sys/stat.h>
#include "messages.h"
#include "buflect.h"
#include "fmail.h"
#include "trtentete.h"
#include "modepage.h"
#include "carspe.h"
#include "szchemin.h"


// mettez le #define qui suit en commentaire si vous ne
// voulez aucune correction du fuseau horaire sur les mails
#define correct_fh


#define max_nbfic_ini 500  // nombre max initial de fichiers mail

#define largeurmax    120  // largeur maximale d'une ligne affiche

// largeur ou largeur maximum des diffrents champs d'une ligne
#define sz_date        14
#define szmax_nom     ((largeurmax / 4) - 1)
#define szmax_sujet   (largeurmax - szmax_nom - sz_date - 2)

#define an_pivot       90  // l'anne des mails les plus vieux sur 2 chiffres


/* type de donnes */
typedef struct
{
    char nomfic [12];
    long ordre;
    char nom [szmax_nom];
    char sujet [szmax_sujet];
    char date [sz_date];
} descmail;


/* prototypes */
void lister_mails ();
int  memfic (char *nomfic);
void lect_fuseau_h ();
void corrige_fh (int *jour, int *mois, int *an, int *heure,
                                       int *min, int decal);
int  ajoutlistepossible ();
void tridate ();
void tridate_inv ();
void navigation ();
void affligne (int numlig);
void suprefmail ();
void majliste_mails ();
void recherche (int sens);
int  dansligne (char *chaine, int numlig);


/* variable globale au source
   (pour viter des tonnes de passages de paramtres) */

descmail **fichmail; // noms des fichiers mail
int nbmaxfic;        // nombre maximun de fichiers mails mmoriss
int fh_loc;          // fuseau horaire local
int test_xorig;      // traitement ou non du champ X-Original-From


/* programme principal */

int main (int nbarg, char *varg[])
{
    int opt_inv;     // option d'inversion de l'ordre d'affichage
    char fic_cmd_trad [20];  // nom fichier qui peut avoir t cr par vmailfic


    // rcupration du nom de l'excutable
    memcom (*varg);

    // mmorisation de l'option -i (ou -r) ventuelle
    if (--nbarg > 0  && ((strcmp (varg [1], "-i") == 0)
                      || (strcmp (varg [1], "-r") == 0)))
    {
        opt_inv = 1;

        // passage  l'argument suivant
        varg ++;
        nbarg --;
    }
    else
        opt_inv = 0;

    // rcupration ventuelle du nom du rpertoire des mails
    if (nbarg == 1)
    {
        // positionnement dans ce rpertoire
        if (chdir (varg [1]) < 0)
        {
            // "Rpertoire %s inexistant"
            aff_err_arg ("REPERT_INEXISTANT", varg [1]);
            return (0);
        }
    }

    // si aucune erreur dans la liste des paramtres
    if (nbarg < 2)
    {
        // fixer la taille initiale de la liste des fichiers mail
        nbmaxfic = max_nbfic_ini;

        // allouer la liste des fichiers mail
        fichmail = malloc (nbmaxfic * sizeof (descmail *));

        // vrification allocation
        if (! fichmail)
            // "Manque de place mmoire, le logiciel %s ne peut fonctionner"
            errfatale ("MANQUE_MEMOIRE", nomcom ());

#ifdef correct_fh
        // rcuprer et mmoriser le fuseau horaire local
        lect_fuseau_h ();
#endif

        // rcupration de la liste des fichiers mail
        lister_mails ();

        // si des fichiers mails ont t trouvs
        if (nb_lignes > 0)
        {
            // les ordonner par date
            if (opt_inv)
                tridate_inv ();
            else
                tridate ();

            // gnrer le nom d'un fichier destin  contenir une commande de
            // traduction. Il pourra tre cr lors d'un appel de vmailfic
            // envoyer ce nom de fichier aux processus fils via une variable
            // d'environement.
            sprintf (fic_cmd_trad, "/tmp/cmd-trad-%04X", getpid ());
            setenv ("libremail-cmd-trad", fic_cmd_trad, 1);

            // remarque : on procde ainsi pour pouvoir rutiliser le mme
            // choix de traduction pour tous les mails du rpertoire.
            // avec une version de gnu/linux assez ancienne, on pouvait
            // fabriquer ce nom de fichier dans les processus fils en utilisant
            // la fonction getppid, mais a ne fonctionne plus sur les
            // distributions rcentes.

            // afficher le caractristiques des mails et naviguer dans la liste
            navigation ();

            // dtruire le fichier fic_cmd_trad s'il a t cr
            unlink (fic_cmd_trad);
        }
        else
            // sinon, pas ncessaire de rester dans l'outil
            // "pas de mails dans ce rpertoire"
            affiche_msg ("REPERT_SANS_MAIL");
    }
    else
        // "Syntaxe : %s [-i] [rpertoire]"
        psyntaxe ("SYNT_VMAILSJ");


    // pour faire plaisir  gcc qui veut une fonction main de type int
    return (0);
}


/* rcupre les caractristiques des mails */

void lister_mails ()
{
    DIR    *repert;
    struct dirent *entree;
    struct stat   descfic;
    int    errmem;
    char   *varenv_xorig;


    // initialisation
    nb_lignes = 0;
    errmem = 0;

    // dterminer si on va traiter les champs X-Original-From
    varenv_xorig = getenv ("libremail_xorig");

    if (varenv_xorig)
        test_xorig = (tolower (*varenv_xorig) != 'n');
    else
        test_xorig = 1;

    // accder au rpertoire
    repert = opendir (".");

    // lire un nom de fichier
    entree = readdir (repert);

    // tant qu'il y a des noms de fichier  lire et qu'on peut les traiter
    while (entree && ! errmem)
    {
        // mmoriser le fichier si son nom convient
        if (strlen (entree->d_name) == 8 ||
           (strlen (entree->d_name) == 10 && entree->d_name [8] == '.'))
        {
            // et si ce n'est pas un rpertoire
            stat (entree->d_name, &descfic);

            if ((descfic.st_mode & S_IFMT) == S_IFREG)
                errmem = memfic (entree->d_name);
        }

        // et lire le nom du fichier suivant
        entree = readdir (repert);
    }

    // avertissement si on a atteint les limites de stockage
    if (errmem)
    {
        effpage ();

        // "Manque de place mmoire, liste des mails tronque"
        affiche_msg_nocr ("DEBORD_MEM_TRONCAT");

        attendre (3);
    }
}


/* mmorisation des caractristiques d'un mail
   retourne 1 si problme d'espace mmoire, 0 dans tous les autres cas */

int memfic (char *nomfic)
{
    // buffers pour mmoriser les caractristiques principales du message
    char bufFromTo [80], bufSubject [80], bufDate [80];

    // les noms de mois dans les champs Date: des mails
    static char *mois [] = { "XXX", "Jan", "Feb", "Mar", "Apr", "May", "Jun",
                                    "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };

    descmail *mailcourant;  // la description en cours de gnration
    int jour, numois, an, heure, min;  // date et heure du mail
    int fhoraire, decalh; // fuseau horaire et dcalage horaire
    int affichedest; // indique si on doit afficher le champ destinataire
    int i, j;        // simples compteurs


    // ouverture du fichier mail
    fmail = fopen (nomfic, "r");

    // sortie si erreur imprvue
    if (fmail == 0)
    {
        affpage ();

        // "Fichier %s protg en lecture"
        aff_err_arg ("ACCES_FICH_LECT", nomfic);
        montecurs ();

        attendre (2);

        return (0);  // erreur non remonte
    }

    // dterminer le champ  afficher entre expditeur et destinataire
    affichedest = (tolower (*nomfic) == 'e') || (tolower (*nomfic) == 's');

    // initialisation
    bufFromTo [0]  = '\0';
    bufSubject [0] = '\0';
    bufDate [0]    = '\0';

    // tantqu'on peut lire une ligne de l'entte du mail
    while (lire_fmail () && buf_lect [0] != '\0')
    {
        // si la ligne correspond au champ sujet
        if (start ("Subject"))
        {
            // mmoriser la ligne
            memconvbuf (bufSubject);

            // on va tester la ligne qui suit
            lire_fmail ();
            majlignentete ();

            // si c'est encore une ligne du sujet
            if (*buf_lect == ' ' || *buf_lect == '\t')
            {
                // initialisation compteurs
                i = 1;
                j = strlen (bufSubject);

                // complter le sujet du mail
                while (buf_lect [i] && j < sizeof (bufSubject) - 1)
                    bufSubject [j++] = buf_lect [i++];

                // terminer la chaine de caractres
                bufSubject [j] = '\0';
            }
        }

        // mmorisation des autres caractristiques importantes
        if (start ("Date"))
            membuf (bufDate);
        else if (start ("From") && ! affichedest && ! *bufFromTo)
            memconvbuf (bufFromTo);
        else if (start ("X-Original-From") && test_xorig)
            memconvbuf (bufFromTo);
        else if (start ("To") && affichedest)
            memconvbuf (bufFromTo);
    }

    // librer le fichier mail
    fclose (fmail);

    // si l'on n'a pas trouv de ligne Date: From: ou Subject:
    if (! *bufDate && ! *bufFromTo && ! *bufSubject)
        return (0);   // ce n'est pas un fichier mail

    // si on ne peut pas mmoriser la description dans la liste
    if (! ajoutlistepossible ())
        return (1);   // on travaillera avec une liste partielle

    // allouer de l'espace mmoire pour stocker les caractristiques du mail
    mailcourant = (descmail *) malloc (sizeof (descmail));

    // si manque de place mmoire
    if (! mailcourant)
        return (1);   // on travaillera avec une liste partielle

    // mmoriser le nom du fichier
    strcpy (mailcourant->nomfic, nomfic);


    // mmorisation du nom de l'expditeur ou du destinataire du message
    j = 0;

    if (*bufFromTo)
    {
        // sauter le mot cl  From: , To:  ou  X-Original-From:
        if (affichedest)
            i = 4;
        else if (tolower (*bufFromTo) == 'x')
            i = 16;
        else
            i = 6;

        // se positionner sur le 1er caractre significatif du nom
        while (bufFromTo [i] == '"' || bufFromTo [i] == ' ')
            i++;

        // mmorisation du nom
        mailcourant->nom [j++] = bufFromTo [i++];

        while (j < szmax_nom && bufFromTo [i] && bufFromTo [i] != '<'
                     && bufFromTo [i] != '"' && bufFromTo [i] != '=')
            mailcourant->nom [j++] = bufFromTo [i++];
    }

    // complter le nom par des blancs
    while (j < szmax_nom)
        mailcourant->nom [j++] = ' ';


    // mmorisation du sujet du message
    j = 0;

    if (*bufSubject)
    {
        // sauter le mot cl  Subject:
        i = 8;

        // se positionner sur le 1er caractre significatif du sujet
        while (bufSubject [i] == ' ')
            i++;

        // mmoriser le sujet
        while (j < szmax_sujet && bufSubject [i])
            mailcourant->sujet [j++] = bufSubject [i++];
    }

    // complter le sujet par des blancs
    while (j < szmax_sujet)
        mailcourant->sujet [j++] = ' ';


    // mmorisation de la date et de l'heure du message
    if (*bufDate)
    {
        // sauter le mot cl  Date:
        i = 6;

        // rcuprer le jour
        while (bufDate [i] < '0' || bufDate [i] > '9')
            i++;

        jour = atoi (bufDate + i);

        // se positionner sur le mois
        while (bufDate [i] > ' ')
            i++;

        i++;

        // rcuprer le numro du mois
        numois = 12;

        while (numois > 0 && (toupper (bufDate [i])  != mois [numois][0]
                           || tolower (bufDate [i+1]) != mois [numois][1]
                           || tolower (bufDate [i+2]) != mois [numois][2]))
            numois--;

        // rcuprer l'anne sur 2 chiffres
        while (bufDate [i] < '0' || bufDate [i] > '9')
            i++;

        an = atoi (bufDate + i) % 100;

        // rcuprer l'heure et les minutes
        while (bufDate [i] > ' ')
            i++;

        heure = atoi (bufDate + i);
        min = atoi (bufDate + i + 4);

#ifdef correct_fh
        // se positionner sur le fuseau horaire
        while (bufDate [i] && bufDate [i] != '+' && bufDate [i] != '-')
            i++;

        // rcuprer le fuseau horaire
        if (bufDate [i])
        {
            // on traitera les dcalages sur les heures et les minutes
            fhoraire = atoi (bufDate + i);

            // s'il faut faire une correction de fuseau horaire
            if (fhoraire != fh_loc && fh_loc > -2400)
                // la faire
                corrige_fh (&jour, &numois, &an, &heure, &min,
                                                 fh_loc - fhoraire);
        }
#endif

        // mmoriser la date sous forme de chaine de caractres
        sprintf (mailcourant->date, "%02d/%02d/%02d %02d:%02d",
                                    jour, numois, an, heure, min);

        // les annes 90  99 sont considres du sicle pass
        if (an >= an_pivot)
            an = an - 100;

        // mmoriser aussi la date sous forme numrique pour tri ultrieur
        /* on tient compte du rglage le plus frquent sur les ordinateurs
           europens : l'ordinateur est rgl sur l'heure locale */

        mailcourant->ordre = ((((((((long) an * 12) + numois) * 31)
                                + jour) * 24) + heure) * 60) + min;
    }
    else
    {
        mailcourant->date [0] = '\0';
        mailcourant->ordre = 0;
    }

    // mmoriser la description dans la liste
    fichmail [nb_lignes++] = mailcourant;

    // tout s'est bien droul
    return (0);
}


/* rcuprer et mmoriser le fuseau horaire local */

void lect_fuseau_h ()
{
    char commande [26];
    char *nomfic;
    FILE *fcaract;


    // initialisation avec une valeur "impossible"
    fh_loc = -9999;

    // rcuprer le dcalage de l'heure locale par rapport au temps universel
    sprintf (commande, "date +%%z > /tmp/fich%04X", getpid ());
    system (commande);

    // ouvrir le fichier cr par la commande  date
    nomfic = commande + 11;
    fcaract = fopen (nomfic, "r");

    // si ouverture russie
    if (fcaract)
    {
        // lire le fuseau horaire sous forme numrique
        fscanf (fcaract, "%d", &fh_loc);

        // fermer et dtruire le fichier
        fclose (fcaract);
        unlink (nomfic);
    }
}


/* corriger le dcalage horaire d'un mail */

void corrige_fh (int *jour, int *mois, int *an, int *heure, int *min, int decal)
{
    // nombre de jours des diffrents mois de l'anne
    // le mois 0 correspond  dcembre de l'anne prcdente
    int dureemois [] = {31, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};


    // correction des minutes
    *min = *min + (decal % 100);

    // changement d'heure ventuel
    if (*min < 0)
    {
        *min = *min + 60;
        (*heure) --;
    }
    else if (*min >= 60)
    {
        *min = *min - 60;
        (*heure) ++;
    }

    // correction de l'heure
    *heure = *heure + (decal / 100);

    // si retour au jour prcdent
    if (*heure < 0)
    {
        // corriger l'heure et le jour du mois
        *heure = *heure + 24;
        *jour = *jour - 1;

        // si retour au mois prcdent
        if (*jour < 1)
        {
            // viter un plantage si le mois n'a pas pu tre dcod
            if (*mois < 1)
                return;

            // corriger le mois et le jour
            *mois = *mois - 1;
            *jour = dureemois [*mois];

            // si retour  l'anne prcdente
            if (! *mois)
            {
                // corriger le mois et l'anne
                *mois = 12;
                *an = *an - 1;
            }
            // sinon si mois de fvrier d'une anne bisextile
            else if (*mois == 2 && (*an % 4) == 0)
                // corriger le jour
                *jour = *jour + 1;
        }
    }
    // sinon si passage au jour suivant
    else if (*heure > 23)
    {
        // corriger l'heure et le jour du mois
        *heure = *heure - 24;
        *jour = *jour + 1;

        // si passage au mois suivant
        if (*jour > dureemois [*mois])
        {
            // test suplmentaire par rapport au 29 fvrier
            if (*mois != 2 || (*an % 4) || *jour > 29)
            {
                // corriger le mois et le jour
                *mois = *mois + 1;
                *jour = 1;

                // si changement d'anne
                if (*mois > 12)
                {
                    // corriger le mois et l'anne
                    *mois = 1;
                    *an = *an + 1;
                }
            }
        }
    }
}


/* vrifie si l'on peut insrer un lment de plus dans le tableau
   fichmail, et redimmensionne ce tableau si ncessaire */

int ajoutlistepossible ()
{
    descmail **nouvtableau; // adresse du tableau de remplacement
    int  nouvtaille;        // et sa taille
    int  element;           // compteur : numro d'lment dans les tableaux


    // cas simple : il reste au moins une place de libre dans le tableau
    if (nb_lignes + 1 < nbmaxfic)
        return (1);

    // calculer la nouvelle taille du tableau
    // l'augmentation est alternativement de 50 % ou 33 % de manire
    //  ce que la taille double aprs 2 rallocations
    if (nbmaxfic % 3)
        nouvtaille = nbmaxfic + (nbmaxfic / 2);
    else
        nouvtaille = nbmaxfic + (nbmaxfic / 3);

    // allocation mmoire
    nouvtableau = malloc (nouvtaille * sizeof (descmail *));

    // vrification allocation
    if (nouvtableau)
    {
        // copie du contenu de l'ancien tableau dans le nouveau
        for (element = 0; element < nbmaxfic; element ++)
            nouvtableau [element] = fichmail [element];

        // destruction de l'ancien tableau
        free (fichmail);

        // que l'on remplace par le nouveau
        fichmail = nouvtableau;
        nbmaxfic = nouvtaille;
    }

    // retourne le rsultat de la possibilit d'insertion d'lments
    return (nb_lignes + 1 < nbmaxfic);
}


/* tri des mails par date et heure croissantes */

void tridate ()
{
    descmail *tampon; // adresse d'une description de mail pour permutation
    int  i;           // indice dans la liste


    // initialisation
    i = 0;

    while (i < nb_lignes - 1)
    {
        // on compare 2 lments conscutifs
        if (fichmail [i]->ordre > fichmail [i+1]-> ordre)
        {
            // interversion de 2 enregistrements
            tampon = fichmail [i];
            fichmail [i] = fichmail [i+1];
            fichmail [i+1] = tampon;

            // ce qui peut supposer de comparer un nom dplac avec le prcdent
            if (i > 0)
                i--;
            else
                i++;
        }
        else
            i++;    // on progresse dans la liste
    }
}


/* tri des mails par date et heure dcroissantes */

void tridate_inv ()
{
    descmail *tampon; // adresse d'une description de mail pour permutation
    int  i;           // indice dans la liste


    // initialisation
    i = 0;

    while (i < nb_lignes - 1)
    {
        // on compare 2 lments conscutifs
        if (fichmail [i]->ordre < fichmail [i+1]-> ordre)
        {
            // interversion de 2 enregistrements
            tampon = fichmail [i];
            fichmail [i] = fichmail [i+1];
            fichmail [i+1] = tampon;

            // ce qui peut supposer de comparer un nom dplac avec le prcdent
            if (i > 0)
                i--;
            else
                i++;
        }
        else
            i++;    // on progresse dans la liste
    }
}


/* affiche la liste des mails du rpertoire et permet de
   la parcourir mme si elle tient sur plusieurs pages */

void navigation ()
{
    int  car;          // caractre tap au clavier
    int  erreurs;      // nombre frappes caractres inconnus comme commandes
    int  choix;        // choix de ce qu'on veut imprimer
    char curdir [szchemin]; // rpertoire courant
    int  dans_mailenv; // indicateur : on est dans le rpertoire d'envoi
    char *editeur;     // diteur utilis pour modifier les mails
    // chaine de caractres utilise pour rebaptiser,
    // mettre  la poubelle ou imprimer un fichier mail
    char tampon [szchemin + 16];


    // configurer la liaison clavier pour lecture directe avec timeout
    mode_raw ();

    // initialisation
    lignecran = 1;
    lignecour = 0;
    erreurs = 0;
    affpage ();

    // rcuprer le nom du rpertoire courant
    getcwd (curdir, szchemin);

    // tester si on est dans le rpertoire d'envoi des mails
    if (getenv ("mailenv"))
        dans_mailenv = (strcmp (curdir, getenv ("mailenv")) == 0);
    else
        dans_mailenv = 0;

    do
    {
        // lire et traiter une touche du clavier
        car = leccar ();

        switch (car)
        {                  // dplacement dans la liste des mails
            case MONTE   : monte (1);
                           erreurs = 0;
                           break;

            case DESCEND : descend (1);
                           erreurs = 0;
                           break;

            case PAGEUP  : monte (lignepage + lignecran - 2);
                           erreurs = 0;
                           break;

            case PAGEDOWN: descend (2 * lignepage - lignecran - 2);
                           erreurs = 0;
                           break;

            case HOME    :
            case HOMEg   : monte (lignecour);
                           erreurs = 0;
                           break;

            case FIN     :
            case FINg    : descend (nb_lignes - lignecour - 1);
                           erreurs = 0;
                           break;

            case 0       : // aide si trop d'erreurs ou  la demande
            case F1      : effpage ();
                           // "Touches utilisables :\n"
                           // "flches Pageup, Pagedown, Home et Fin"
                           // "pour se dplacer d'une ou plusieurs lignes"
                           // "v ou l ou entre pour Voir (Lire) un mail"
                           // "r pour y Rpondre, t pour le Transfrer"
                           // "s pour le Supprimer, m pour Modifier avant envoi"
                           // "u pour restaurer un mail dans la poubelle"
                           // "j pour Joindre des fichiers  un mail  envoyer"
                           // "n ou Inser pour en crer un Nouveau"
                           // "o pour voir le fichier Original (non converti)"
                           // "i pour Identifier le fichier mail"
                           // "p pour imPrimer un mail (ou la liste complte)"
                           // "/ ou ? pour rechercher une chaine dans la liste"
                           // "Control L pour rafficher la page"
                           // "Esc pour sortir de ce programme\n"
                           // "Appuyer sur une touche pour continuer"
                           affiche_msg ("AIDE_CHOIX_ADR-1");
                           affiche_msg ("AIDE_CHOIX_ADR-2");
                           affiche_msg ("AIDE_CHOIX_ADR-3");
                           affiche_msg ("AIDE_VMAILSJ-1");
                           affiche_msg ("AIDE_VMAILSJ-2");
                           affiche_msg ("AIDE_VMAILFIC-2");
                           affiche_msg ("AIDE_VMAILFIC-3");
                           affiche_msg ("AIDE_VMAILFIC_SJ2");
                           affiche_msg ("AIDE_VMAILDIR_SJ");
                           affiche_msg ("AIDE_VMAILSJ-3");
                           affiche_msg ("AIDE_VMAILFIC_SJ1");
                           affiche_msg ("AIDE_VMAILSJ-4");
                           affiche_msg ("AIDE_VMAILSJ-5");
                           affiche_msg ("AIDE_CHOIX_ADR-7");
                           affiche_msg ("AIDE_VMAIL");
                           affiche_msg ("ATTENTE_CLAVIER");
                           leccar ();
                           affpage ();
                           erreurs = 0;
                           break;

            case 'l'     :
            case 'v'     : // on va visualiser un mail
            case '\n'    : execom ("vmailfic", fichmail [lignecour]->nomfic);

                           // pas de surbrillance pour les mails dj visualiss
                           // (le test de la lettre  n  comme 10me caractre
                           // n'est utile que pour assurer la compatibilit en
                           // lecture avec Micro$oft Internet mail)
                           if (fichmail [lignecour]->nomfic [8] == '.'
                                && tolower(fichmail[lignecour]->nomfic[9])== 'n'
                                && access (fichmail [lignecour]->nomfic, 0) < 0)
                               fichmail [lignecour]->nomfic [8] = '\0';

                           // si le mail a t supprim
                           if (access (fichmail [lignecour]->nomfic, 0) < 0)
                               // on l'enlve de la liste  afficher
                               suprefmail ();

                           // on peut rafficher la page
            case CTRL    : affpage ();
                           erreurs = 0;
                           break;

                           // identifie le fichier mail (donne son nom)
            case 'i'     : effpage ();
                           // "\nFichier : %s/%s\n\n"
                           printf (message ("AFF_CHEMFICH"), curdir,
                                             fichmail [lignecour]->nomfic);

                           // "Appuyer sur une touche pour continuer"
                           affiche_msg_nocr ("ATTENTE_CLAVIER");

                           leccar ();
                           affpage ();
                           erreurs = 0;
                           break;

                           // voir le fichier mail original sans conversion
            case 'o'     : execom ("less -r", fichmail [lignecour]->nomfic);
                           affpage ();
                           erreurs = 0;
                           break;

            case INSERT  : // cration d'un nouveau mail
            case 'n'     : execom ("cremail", "");

                           // si on est dans le rpertoire des mails  envoyer
                           if (dans_mailenv)
                               // ractualiser la liste des mails affichs
                               majliste_mails ();

                           affpage ();
                           erreurs = 0;
                           break;

                           // rponse  un mail
            case 'r'     : execom ("repmail", fichmail [lignecour]->nomfic);

                           // si on est dans le rpertoire des mails  envoyer
                           if (dans_mailenv)
                               // ractualiser la liste des mails affichs
                               majliste_mails ();

                           affpage ();
                           erreurs = 0;
                           break;

                           // transfert d'un mail
            case 't'     : execom ("trsfmail", fichmail [lignecour]->nomfic);

                           // si on est dans le rpertoire des mails  envoyer
                           if (dans_mailenv)
                               // ractualiser la liste des mails affichs
                               majliste_mails ();

                           affpage ();
                           erreurs = 0;
                           break;

      //    case SUPR    : // suppression d'un mail
            case 's'     : // rcuprer le rpertoire des mails supprims
                           if (getenv ("mailpoub"))
                               strcpy (tampon, getenv ("mailpoub"));
                           else
                               *tampon = '\0';

                           // si c'est le rpertoire courant
                           if (strcmp (curdir, tampon) == 0)
                           {
                               // supprimer le fichier mail
                               unlink (fichmail [lignecour]->nomfic);

                               // supprimer le mail de la liste  afficher
                               suprefmail ();
                           }
                           // sinon (cas gnral)
                           else if (*tampon)
                           {
                               // crer le rpertoire poubelle si ncessaire
                               mkdir (tampon, 0755);

                               // et y mettre le fichier mail
                               strcat (tampon, "/");
                               strcat (tampon, fichmail [lignecour]->nomfic);
                               rename (fichmail [lignecour]->nomfic, tampon);

                               // supprimer le mail de la liste  afficher
                               suprefmail ();
                           }
                           // sinon (racine des mails inconnue)
                           else
                           {
                               effpage ();

                            // "Racine des mails inconnue, pas de suppression"
                               affiche_msg_nocr ("REP_RACINE_INCONNU");

                               attendre (2);
                           }

                           affpage ();
                           erreurs = 0;
                           break;

                           // restauration d'un mail
            case 'u'     : // si on est dans le rpertoire poubelle
                           if (strcmp (curdir, getenv ("mailpoub")) == 0)
                           {
                               // dplacer le fichier mail dans entree
                               sprintf (tampon, "../%s/%s",
                          ficdir ("DIR_ENTREE"), fichmail [lignecour]->nomfic);
                               rename (fichmail [lignecour]->nomfic, tampon);

                               // supprimer le mail de la liste  afficher
                               suprefmail ();
                           }
                           // sinon, message d'erreur
                           else
                           {
                               effpage ();

                    // "Seuls les mails dans la poubelle peuvent tre restaurs"
                               affiche_msg_nocr ("MAIL_NON_RESTAUR");

                               attendre (2);
                           }

                           affpage ();
                           erreurs = 0;
                           break;

                           // modification d'un mail  envoyer
            case 'm'     : // si on est dans le rpertoire des mails  envoyer
                           if (dans_mailenv)
                           {
                               // modifier le mail
                               editeur = getenv ("EDITOR");

                               if (editeur)
                                   execom (editeur,fichmail[lignecour]->nomfic);
                               else
                                   execom ("vi", fichmail [lignecour]->nomfic);
                           }
                           else
                           {
                               // sinon, message d'erreur
                               effpage ();

                   // "Seuls les mails en attente d'envoi peuvent tre modifis"
                               affiche_msg_nocr ("MAIL_NON_MODIF");

                               attendre (2);
                           }

                           affpage ();
                           erreurs = 0;
                           break;

                           // impression de la liste ou du mail courant
            case 'p'     : // choix de ce qu'on veut imprimer
                           putchar ('\n');
                           effpage ();
                           // "1) impression de la liste des mails"
                           // "2) impression du mail de la ligne courante"
                           // "Votre choix ? "
                           affiche_msg ("CHOIX_IMPR-1");
                           affiche_msg ("CHOIX_IMPR-2");
                           affiche_msg_nocr ("VOTRE_CHOIX");

                           choix = leccar () - '0';

                           // lancer la commande d'impression correspondante
                           if (choix == 1)
                               system ("sjfmails . | lp");
                           else if (choix == 2)
                           {
                               sprintf (tampon, "voirfmail %s | lp",
                                        fichmail [lignecour]->nomfic);
                               system (tampon);
                           }
                           else
                           {
                               // "Choix erronn"
                               affiche_msg_nocr ("CHOIX_IMPR_ERR");

                               attendre (2);
                           }

                           affpage ();
                           erreurs = 0;
                           break;

                           // rajout de fichiers joints  un mail  envoyer
            case 'j'     : // si on est dans le rpertoire des mails  envoyer
                           if (dans_mailenv)
                               // rajouter des fichiers joints
                               execom ("joindre", fichmail [lignecour]->nomfic);
                           else
                           {
                               // sinon, message d'erreur
                               effpage ();

                   // "Seuls les mails en attente d'envoi peuvent tre modifis"
                               affiche_msg_nocr ("MAIL_NON_MODIF");

                               attendre (2);
                           }

                           affpage ();
                           erreurs = 0;
                           break;

            case '/'     : // recherche d'une chaine de caractres
            case '?'     : recherche (car);
                           erreurs = 0;

                           // sortie du programme
            case ESC     : break;

                           // sortie aussi sur :q
            case ':'     : car = leccar ();

                           if (car == 'q')
                           {
                               car = ESC;
                               break;
                           }

            default      : putchar (7);  // bip

                           // si trop d'erreurs, afficher l'aide
                           if (++erreurs == 5)
                               ungetc (0, stdin);  // on ira sur l'aide
        }
    }
    while (car != ESC && nb_lignes > 0);

    // descendre en bas de page (pour sortir proprement du programme)
    while (lignecran++ < lignepage && lignecour++ < nb_lignes)
        putchar ('\n');

    // retour  la configuration standard du clavier
    mode_normal ();
}


/* affiche les caractristiques d'un mail */

void affligne (int numlig)
{
    // la largeur des diffrents champs affichs est calcule
    //  partir de la largeur de la fentre d'affichage
    // on utilise des variables statiques pour ne faire le
    // calcul qu'en cas de ncessit
    static int old_colonpage = -1; // pour dtecter changement largeur de page
    static int sz_nom, sz_sujet;   // longueur des diffrentes zones affiches
    static char car_nonlu = 0;     // caractre indiquant les mails non lus
    int tot;    // pour calcul
    int i, j;   // simples compteurs


    // si la largeur de la zone d'affichage a chang
    if (colonpage != old_colonpage)
    {
        // rpartir la largeur d'affichage entre les champs
        sz_nom = (colonpage / 4) - 1;

        if (sz_nom > szmax_nom)
            sz_nom = szmax_nom;

        sz_sujet = colonpage - sz_date - sz_nom - 2;

        if (sz_sujet > szmax_sujet)
            sz_sujet = szmax_sujet;

        // cette mmorisation vitera de refaire le calcul  chaque ligne
        old_colonpage = colonpage;

        // si la variable d'environnement libremail_new est utilise
        if (getenv ("libremail_new"))
        {
            // mmoriser ce caractre
            car_nonlu = *getenv ("libremail_new");

            // corriger la largeur d'affichage des 2 premiers champs
            sz_nom --;
            sz_sujet --;
        }
    }

    // afficher en surbrillance les mails non lus
    if (fichmail [numlig]->nomfic [8] == '.' &&
           tolower (fichmail [numlig]->nomfic [9]) == 'n')
    {
        clair ();

        // si ncessaire avec le caractre d'identification des mails non lus
        if (car_nonlu)
        {
            putchar (car_nonlu);
            putchar (' ');
        }
    }
    else if (car_nonlu)
    {
        putchar (' ');
        putchar (' ');
    }

    // si affichage avec un jeu de caractres ISO-8859
    if (! util_utf8 ())
    {
        // affichage du nom de l'expditeur
        for (i = 0; i < sz_nom; i++)
            putchar (fichmail [numlig]->nom [i]);

        putchar (' ');

        // affichage du sujet
        for (i = 0; i < sz_sujet; i++)
        {
            // si prsence d'une tabulation
            if (fichmail [numlig]->sujet [i] == '\t')
                // afficher un blanc pour une meilleure mise en page
                putchar (' ');
            else
                // sinon affichage du caractre
                putchar (fichmail [numlig]->sujet [i]);
        }
    }
    // sinon (affichage avec le jeu de caractres UTF-8)
    else
    {
        // initialisation
        j = 0;

        // affichage du nom de l'expditeur
        for (i = 0; i < sz_nom + j && i < szmax_nom; i++)
        {
            putchar (fichmail [numlig]->nom [i]);

            // mais certains caractres ne seront pas compts
            if ((fichmail [numlig]->nom [i] & 0xC0) == 0x80)
                j++;
        }

        // complt par des blancs
        while (i++ <= sz_nom + j)
            putchar (' ');

        // rinitialisation
        j = 0;

        // affichage du sujet
        for (i = 0; i < sz_sujet + j && i < szmax_sujet; i++)
        {
            // si prsence d'une tabulation
            if (fichmail [numlig]->sujet [i] == '\t')
                // afficher un blanc pour une meilleure mise en page
                putchar (' ');
            // sinon
            else
            {
                // affichage du caractre
                putchar (fichmail [numlig]->sujet [i]);

                // mais certains caractres ne seront pas compts
                if ((fichmail [numlig]->sujet [i] & 0xC0) == 0x80)
                    j++;
            }
        }

        // complt par des blancs
        while (i++ < sz_sujet + j)
            putchar (' ');
    }

    putchar (' ');

    // affichage de la date et de l'heure (sans fuseau horaire)
    printf ("%s", fichmail [numlig]->date);

    // retour  l'affichage normal
    lumnor ();
}



/* supprime de la liste un mail que l'on vient de dtruire */

void suprefmail ()
{
    int i;  // compteur


    // on l'enlve de la liste  afficher
    for (i = lignecour + 1; i < nb_lignes; i++)
        fichmail [i - 1] = fichmail [i];

    nb_lignes --;

    // on reste positionn sur un mail voisin
    if (lignecour == nb_lignes)
        monte (1);
}



/* ractualiser l'affichage de la liste des mails

   Cette fonction ne sert que dans le cas particulier d'un
   mail rajout alors qu'on est dans le rpertoire d'envoi.
   La mthode de mise  jour n'a pas besoin d'tre optimise.
   On se contente d'effacer l'ancienne liste et de la recrer. */

void majliste_mails ()
{
    // librer la mmoire occupe par l'ancienne liste
    while (nb_lignes > 0)
        free (fichmail [--nb_lignes]);

    // rcuprer la liste des fichiers mail  envoyer
    lister_mails ();

    // et les trier par date
    tridate ();
}



/* recherche d'une chaine de caractres dans la liste des mails */

void recherche (int sens)
{
    // chaine  rechercher, on utilise une variable statique
    // pour garder sa valeur aux appels suivants de la fonction
    static char chaine [80];
    int car;    // caractre lu au clavier
    int limite; // ligne marquant une extrmit de la liste
    int trouve; // indicateur : ligne contenant la chaine trouve
    int i, j;   // compteurs


    // descendre en bas de page et effacer la dernire ligne
    baspage ();
    effligne ();

    // saisir la chaine  rechercher
    putchar (sens);
    car = leccar ();

    // si chaine vide, on conserve la prcdente
    if (car != '\n')
    {
        // sinon on mmorise la nouvelle chaine
        i = 0;

        do
        {
            // cas particulier : caractre d'effacement
            if (car == RECULE || car == EFFCAR)
            {
                if (i > 0)
                {
                    putchar (CTRH);
                    putchar (' ');
                    putchar (CTRH);
                    i--;
                }
            }
            // cas gnral : affichage et mmorisation d'un caractre
            else if (i < sizeof (chaine))
            {
                putchar (car);
                chaine [i++] = car;
            }

            // lire le caractre suivant
            car = leccar ();
        }
        // jusqu' ce que chaine entirement saisie
        while (car != '\n');

        // terminer la chaine
        chaine [i] = '\0';
    }

    // initialisation variables pour recherche
    if (sens == '/')
    {
        sens = 1;
        limite = nb_lignes - 1;
    }
    else
    {
        sens = -1;
        limite = 0;
    }

    // dbut de la recherche  partir d'une ligne voisine de la ligne courante
    i = lignecour;

    // rpeter
    do
    {
        // si on n'a pas atteint l'extrmit de la liste des mails
        if (i != limite)
            // descendre ou remonter d'une ligne selon le sens de recherche
            i = i + sens;
        else
            // sinon aller  l'autre extrmit de la liste
            i = nb_lignes - 1 - limite;

        // chercher si la chaine est prsente dans la nouvelle ligne
        trouve = dansligne (chaine, i);
    }
    // jusque chaine de caractres trouve ou toute la liste explore
    while (! dansligne (chaine, i) && i != lignecour);

    // si chaine non trouve
    if (! trouve)
    {
        // afficher un message d'erreur
        // " Chaine non trouve"
        affiche_msg_nocr ("CHAINE_ABSENTE");
        putchar (7);
    }

    // revenir en dbut de ligne
    putchar ('\r');

    // remonter  la ligne sur laquelle on tait
    j = lignepage;

    while (j-- > lignecran)
        montecurs ();

    // si chaine trouve
    if (trouve)
    {
        // se positionner sur la ligne de la chaine s'il y a lieu
        if (i < lignecour)
            monte (lignecour - i);
        else if (i > lignecour)
            descend (i - lignecour);

        // rafficher la ligne contenant la chaine dans une autre couleur
        putseq ("33m");
        affligne (lignecour);
        putchar ('\r');
        lumnor ();
    }
}



/* vrification de la prsence d'une chaine dans la numligime ligne */

int dansligne (char *chaine, int numlig)
{
    char *maligne; // pour viter de trop manipuler des indices de tableau
    int i, j;      // simples compteurs


    // initialisation
    i = 0;

    // le nom de l'expditeur/destinataire, le sujet et la
    // date sont regroups sur une mme chaine de caractres
    maligne = (char*) &fichmail [numlig]->nom;

    // tantque ligne non explore en entier
    while (maligne [i])
    {
        // si caractre courant = premier caractre de la chaine
        if (maligne [i] == *chaine)
        {
            // vrifier la concordance des caractres suivants
            j = 1;

            while (chaine [j] && (chaine [j] == maligne [i+j]))
                j++;

            // si chaine trouve, sortir de la fonction
            if (! chaine [j])
                return 1;
        }

        // sinon, essayer avec la suite de la ligne
        i++;
    }

    // chaine non trouve dans la ligne
    return 0;
}
