/*
    Fichier voirfmail.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence-fr.txt

    Traductions des droits d'utilisation dans les fichiers :
    licence-de.txt , licence-en.txt , licence-es.txt ,
    licence-it.txt , licence-nl.txt , licence-pt.txt ,
    licence-eo.txt , licence-eo-utf.txt

    Droits d'utilisation galement sur la page web :
    http://libremail.tuxfamily.org/voir.php?page=droits


    Ce programme affiche le contenu d'un fichier mail pass en paramtre
    Le chemin d'accs au fichier peut tre absolu ou relatif
*/

#define appli   // pour la dclaration de variables globales  l'application

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "messages.h"
#include "buflect.h"
#include "fmail.h"
#include "encodage.h"
#include "trtentete.h"
#include "trtligne.h"
#include "trtbordure.h"
#include "trtsection.h"


/* prototypes */
void aff_entete ();
void affchamp (long pos_deblig, int conversion);
void aff_mail ();
void aff_texte ();
void genligne (char *chaine);


/* variable globale au source
   (pour viter des tonnes de passages de paramtres) */

int opth;  // option affichage de la section text/html au lieu de text/plain
int optT;  // option d'affichage partiel de l'entte pour les traductions


/* programme principal */

int main (int nbarg, char *varg[])
{
    // rcupration du nom de l'excutable
    memcom (*varg);

    // pas encore d'option mmorise
    opth = 0;
    optT = 0;

    // traitement des options ventuelles de la commande
    while (--nbarg > 1 && varg [1][0] == '-')
    {
        // si option -h ou -H
        if ((varg [1][1] | 0x20) == 'h')
        {
            // mmoriser cette option
            if (varg [1][1] == 'h')
                opth = 1;
            else
                opth = 2;
        }
        // sinon, prendre en compte l'option -T
        else if (varg [1][1] == 'T')
            optT = 1;

        // sinon, option incorrecte
        else
            // "Syntaxe : %s [-(h|H)] [-T] nom_fichier_mail"
            psyntaxe ("SYNT_VOIRFMAIL");

        // un argument trait
        varg ++;
    }

    // controle du nombre d'arguments
    if (nbarg == 1)
    {
        // ouvrir le fichier mail
        fmail = fopen (varg [1], "r");

        // si le fichier a pu tre ouvert
        if (fmail)
        {
            // afficher l'entte du mail puis son contenu
            aff_entete ();

            // et fermer le fichier mail
            fclose (fmail);
        }
        else
            // "Fichier %s non trouv"
            aff_err_arg ("FICH_ABSENT", varg [1]);
    }
    else
        // "Syntaxe : %s [-(h|H)] [-T] nom_fichier_mail"
        psyntaxe ("SYNT_VOIRFMAIL");

    // pour faire plaisir  gcc qui veut une fonction main de type int
    return (0);
}


/* lit le message choisi et l'affiche
   cette fonction traite essentiellement l'entte du mail
   puis appelle aff_mail pour l'affichage du corps du mail */

void aff_entete ()
{
    long pos_deblig;   // position dans le fichier en dbut de ligne
    // position dans le fichier mail des principaux champs de l'entte
    long posDate, posFrom, posXorig, posTo, posCc, posReply, posSubject,
         posContent;
    char *varenv_xorig; // variable d'environnement libremail_xorig
    char xorig;         // information utile de libremail_xorig


    // initialisation
    posDate     = -1;
    posFrom     = -1;
    posXorig    = -1;
    posTo       = -1;
    posCc       = -1;
    posReply    = -1;
    posSubject  = -1;
    posContent  = -1;

    // rcuprer la partie significative de la variable libremail_xorig
    varenv_xorig = getenv ("libremail_xorig");

    if (varenv_xorig)
        xorig = tolower (*varenv_xorig);
    else
        xorig = '\0';

    // lecture de l'entte et mmorisation de la position des champs importants
    do
    {
        // rcuprer la position en dbut de ligne
        pos_deblig = ftell (fmail);

        // lire une ligne de l'entte du message
        lire_fmail ();

        // reprage des champs importants et mmorisation de leur position

        // Le premier champ est un peu pour le fun
        if ((start ("X-Mailer") || start ("User-Agent")) && ! optT)
            puts (buf_lect);      // le mailer est affich directement
        else if (start ("Date"))
            posDate = pos_deblig;
        else if (start ("From"))
            posFrom = pos_deblig;
        else if (start ("X-Original-From") && xorig != 'n')
            posXorig = pos_deblig;
        else if (start ("To"))
            posTo = pos_deblig;
        else if (start ("Cc"))
            posCc = pos_deblig;
        else if (start ("Reply-To"))
            posReply = pos_deblig;
        else if (start ("Subject"))
            posSubject = pos_deblig;
        else if (start ("Content-Type"))
            posContent = pos_deblig;
        else if (start ("Content-Transfer-Encoding"))
            mem_encodage ();
    }
    while (buf_lect [0] != '\0');  // lecture entte termine si ligne vide

    // si mode de fonctionnement standard
    if (! optT)
    {
        // affichage ordonn des champs principaux de l'entte
        if (posDate >= 0)
            affchamp (posDate, 0);
        if (posFrom >= 0 && (posXorig < 0 || xorig == '2'))
            affchamp (posFrom, 1);
        if (posXorig >= 0)
            affchamp (posXorig, 1);
        if (posTo >= 0)
            affchamp (posTo, 1);
        if (posCc >= 0)
            affchamp (posCc, 1);
        if (posReply >= 0)
            affchamp (posReply, 1);
        if (posSubject >= 0)
            affchamp (posSubject, 1);
    }
    // sinon affichage rduit au sujet sans le nom de ce champ
    else if (posSubject >= 0)
        affchamp (posSubject + 8, 1);

    // rcuprer le type principal du message et ses caractristiques
    recup_typeorig (posContent);

    // revenir au dbut du corps du message
    fseek (fmail, pos_deblig, SEEK_SET);
    lire_fmail ();

    // affichage du contenu du mail
    if (ctypeorig & Multipart)
        // il faudra analyser les sections
        aff_mail ();
    else
        // un affichage simple suffit
        aff_texte ();
}


/* affiche un champ de l'entte d'une ou plusieurs lignes
   repr  partir de sa position dans le fichier mail */

void affchamp (long pos_deblig, int conversion)
{
    // se positionner sur la ligne contenant le champ
    fseek (fmail, pos_deblig, SEEK_SET);

    // lire cette ligne
    lire_fmail ();

    // rpter
    do
    {
        // interprter si ncessaire les caractres spciaux
        if (conversion)
            majlignentete ();

        // et afficher la ligne
        printf ("%s", buf_lect);

        // en tenant compte d'un ventuel \n dans le buffer
        if (buf_lect [strlen (buf_lect) - 1] != '\n')
            putchar ('\n');

        // lire la ligne suivante
        lire_fmail ();
    }
    // jusqu' ce qu'on ait trait tout le champ
    while (*buf_lect == ' ' || *buf_lect == '\t');
}


/* affichage du contenu d'un mail en mode multipart */

void aff_mail ()
{
    // mmorisation des modes multipart trouvs
    int multipmixed = 0, multipalter = 0, multiprel = 0, multiprep = 0;


    // mmoriser tous les modes multipart imbriqus
    do
    {
        switch (ctype)
        {
            case MultipMixed : multipmixed = 1;
                               break;

            case MultipAlter : multipalter = 1;
                               break;

            case MultipRep   : multiprep   = 1;
                               break;

            case MultipRel   : multiprel   = 1;
        }

        // passer  la section suivante
        prochaine_section ();

        // rcuprer ses caractristiques
        recup_infos_section ();
    }
    while ((ctype & Multipart) && lire_fmail ());

    // indiquer si le mail peut contenir des pices jointes
    if (multipmixed)
        // "Pice(s) jointes(s) probable(s)"
        affiche_msg ("PJ_PROBABLE");

    // si une section multipart/alternative a t trouve, se positionner
    // (si l'on n'y est pas) sur la section text/plain ou text/html du mail
    if (multipalter)
    {
        if (opth)
            posit_texthtm ();
        else
            posit_texte ();
    }

    // gnrer une ligne de sparation avec l'entte
    putchar ('\n');

    // si structure du mail non conforme, message d'erreur
    if (! lire_fmail ())
    {
        if (opth)
            // "Pas de zone texte html dans ce mail !!!"
            affiche_msg ("MANQUE_ZONE_HTML");
        else
            // "Pas de zone texte dans ce mail !!!"
            affiche_msg ("MANQUE_ZONE_TEXTE");
    }

    // lecture et affichage du corps du message
    do
    {
        // mise en forme de la dernire ligne lue
        majligne ();

        // si traitement des balises html
        if (opth == 2)
        {
            // supprimer les balises html
            sup_balhtm ();

            // et convertir les caractre sous la forme &...;
            conv_carhtm ();
        }

        // si la dernire ligne lue n'est pas vide et pas d'option -H
        if (*buf_lect != '\n' || opth != 2)
            // l'afficher
            printf ("%s", buf_lect);

        // et lecture de la suivante
        if (! lire_fmail ())
            return;   // on sort en fin de fichier

        // si mode multipart/report, on saute les enttes de section
        if (multiprep && nbordures && surbordure ())
        {
            // aller  la prochaine ligne vide
            while (lire_fmail () && *buf_lect)
                ;
        }
    }
    // on s'arrte en fin de fichier, ou sur la prochaine bordure
    while (nbordures == 0 || ! surbordure ());

    // si mode multipart mixed on va lister les pices jointes
    if (ctypeorig == MultipMixed)
        liste_pj ();
}



/* lecture et affichage d'un mail de type text/plain */

void aff_texte ()
{
    // si le mail est au format test/plain
    if (ctypeorig == TextPlain)
        // on ne traitera pas les balises html
        opth = 0;

    // si texte encod base 64
    if (encodage_texte == Base64)
        // gnrer une ligne de sparation avec l'entte
        putchar ('\n');

    // rpter
    do
    {
        // mise en forme de la dernire ligne lue
        majligne ();

        // si traitement des balises html
        if (opth == 2)
        {
            // supprimer les balises html
            sup_balhtm ();

            // et convertir les caractre sous la forme &...;
            conv_carhtm ();
        }

        // si la dernire ligne lue n'est pas vide et pas d'option -H
        if (*buf_lect != '\n' || opth != 2)
            // l'afficher
            printf ("%s", buf_lect);

        // et lecture de la suivante
    }
    // on s'arrte en fin de fichier
    while (lire_fmail ());
}



/* pour compatibilit avec la bibliothque trtsection */

void genligne (char *chaine)
{
    printf ("%s", chaine);
}
